#ifndef _CONNECTION_H
#define _CONNECTION_H

#include<string>
#include<backend/Statement.h>


//Forward references
class otl_connect;
class Statement;
class PreparedStatement;
class ConnectionFactory;

class Connection
{
//A factory that can make a connection
friend class ConnectionFactory;

private:
	/**
	* Makes a connection object
	* \param username 
	* \param passwd 
	* \param dsnSource 
	* \return 
	*/
	Connection(std::string username,std::string passwd,std::string dsnSource);

public:
	/**
	* Destorys the connection object
	* \return 
	*/
	~Connection();

	/**
	* Commit transaction.
	*/
	void commit();

	/**
	* This method is used to create a sql statement
	* \return Statement
	*/
	Statement* createStatement();

	/**
	* Makes a prepared Statement given a sql string
	* \param sqlStatement 
	* \param bufferSize 
	* \return 
	*/
	PreparedStatement* makePreparedStatement(std::string sqlStatement,
											const unsigned int bufferSize);

	static void initialize();

private :
	//Stores the current user name
	std::string userName;
	//Stores the passwd
	std::string passwd;
	//Stores the name of the dsn source
	std::string dsnSource;	
	//Stores the reference to the otl_connect parameter
	otl_connect* db;
};

#endif //_CONNECTION_H