#ifndef _LOADER_H
#define _LOADER_H
#include <DataBaseLibrary/Connection.h>
#include <DataBaseLibrary/ResultSet.h>
#include <DataBaseLibrary/Statement.h>
#include <Record.h>
#include <Iterator.h>
#include <iostream>
#include <cache/Schema.h>

namespace cache
{
	class RecordIterator : public ::Iterator<Record*>
	{		
		friend class Loader;
		//Only the loader can create this class
	private:
		RecordIterator(ResultSet * resultSet,Schema *schema)
		{			
			this->resultSet = resultSet;			
			this->schema = schema;
			this->numOfColumns = resultSet->getNumberOfColumns();
		}
	public:
		/**
		* Creates a new record note this record has to 
		* be deleted
		* \return 
		*/
		Record* get_next()
		{
			Record *record = NULL;
			//-2 for id and pid -4 for descriptor and -2 for height and N
			unsigned int numberOfDimensions = numOfColumns-2-4-2;		
			if(resultSet->next())
			{	
				record = schema->makeRecord(resultSet);			
			} // if(!resultSet->isDone())
			return(record);
		}

		bool isDone()
		{
			return(resultSet->isDone());	
		}

		~RecordIterator()
		{
			delete(this->resultSet);
		}
	private:
		//Stores the result set pointer
		ResultSet* resultSet;		
		//Stores the pointer to the schema
		Schema* schema;
		//Stores the number of columns
		unsigned int numOfColumns;
	};


	class Loader
	{
	public:
		Loader(::Statement *statement,Schema* schema)
		{
			this->statement = statement;
			this->schema = schema;
		}	
		

		/**
		* Loads the complete query in the result set.
		* \param query 
		* \return 
		*/
		::Iterator<Record*>* load(std::string query)
		{
			try{
				ResultSet* resultSet = statement->executeQuery(query);				
				return(new RecordIterator(resultSet,this->schema));

			}	catch(otl_exception& p){ // intercept OTL exceptions
				std::cerr<<p.code<<std::endl; // print out error code
				std::cerr<<p.sqlstate<<std::endl; // print out error SQLSTATE
				std::cerr<<p.msg<<std::endl; // print out error message
				std::cerr<<p.stm_text<<std::endl; // print out SQL that caused the error
				std::cerr<<p.var_info<<std::endl; // print out the variable that causeAd the error
				return(NULL);
			}
		}	
	private:
		//Stores the sql statement
		::Statement *statement;	
		//Stores the schema this class is used to instantiate the record
		Schema *schema;
	};
}
#endif _LOADER_H