/* file: strategyTable.h
 * purpose: Definitions for Strategy Selection Table class
 * revision log:
 */

#include "db_GLOBAL.h"
#define STRATEGYTABLE_MAXSTRATEGIES STRATEGY_MAXID 

                                      /* Strategy Selection Policies ------------------- */
#define STRATEGYTABLE_POLICYBEST    0 /* always select strategy with highest fitness value */
#define STRATEGYTABLE_POLICYPROP    1 /* fitness-proportionate selection */
#define STRATEGYTABLE_NPOLICIES     2 /* number of strategy selection policies listed above */

										        /* Fitness Function ------------------------ */
#define STRATEGYTABLE_FITGLOBAVERESPTIME			0 /* use global ave response time */
#define STRATEGYTABLE_FITEXPAVERESPTIME				1 /* use exp ave response time */
#define STRATEGYTABLE_FITEXPAVEMISCLASS_COST		2 /* use exp ave response time */
#define STRATEGYTABLE_FITGLOBALAVEMISCLASS_COST		3 /* use exp ave response time */
#define STRATEGYTABLE_NFITNESS						4 /* number of fitness functions listed above */

#define STRATEGYTABLE_RANDSEED 459275  /* seed for random number generator */


void strategyTable_Create() ;
 /*	Returns a new strategyTable, or NULL if there isn't enough memory.
  */

void strategyTable_Destroy(void) ;
 /* Frees the memory associated with 'strategyTable'. 
  */

void strategyTable_SetSelectionPolicy(int policy) ;
 /* Sets the Strategy Selection Policy
  */

void strategyTable_SetFitnessFunction(int fitnessFunction) ;
 /* Sets the Fitness Function
  */

void strategyTable_AddStrategy(int strategyId) ;
 /* Adds a strategy to the table
  */

void strategyTable_Print(void) ;
 /* Prints contents of the strategy table
  */

void strategyTable_UpdateTable(int strategyId, float respTime) ;
 /* Update the statistics for the given strategyId 
  */

void strategyTable_UpdateTable2(float respTime) ;
 /* version of strategyTable_UpdateTable() that uses the internally stored strategyId
  */

void strategyTable_UpdateTable3(float respTime, int correctlyPredicted, int misPredicted, int notPredicted) ;
 /* version of strategyTable_UpdateTable() for new measurements
  */

int strategyTable_SelectStrategy(void) ;
 /* Selects a strategy based on the Selection Policy and Fitness Function.
  * output: strategyId of selected strategy
  */

void strategyTable_Test(int testcase) ;
 /* Tests the entire strategyTable class 
  */

void strategyTable_SelectPrefetchers(void);
 /* Hard code the table with the existing prefetchers
  */

int strategyTable_GetSelectedStrategy(void) ;
 /* Return the internally stored selected strategy id
  */
