// SimpleValue.h: interface for the SimpleValue class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __SIMPLEVALUE_H__
#define __SIMPLEVALUE_H__

#include <meta/Interfaces.h>
#include <meta/Value.h>

class Restriction;

class SimpleValue : public Value,
					public ISerializable
{
public:
	SimpleValue();

	SimpleValue(const SimpleValue& sv)
	{
		mTypeName = sv.mTypeName;
		mParentTypeId = sv.mParentTypeId;
		mValue = sv.mValue->clone();
		mTypeId = sv.mTypeId;
	}

	SimpleValue(const std::string& typeName, Type parentType) :
	mTypeName(typeName), mParentTypeId(parentType)
	{}
	
	static Type getTypeId(SimpleValue *val) {
		if ( val->mTypeId.isBad() ) {
			val->mTypeId = Type::registerType(
				val->mParentTypeId, val->mTypeName.c_str(),
				SimpleValue::createValue);
		}
		return val->mTypeId;
	}
	
	static IValue* createValue(const char* str, void* inst) { return new SimpleValue; }
	
	virtual Type getType() { return getTypeId(this); }
	virtual IValue* clone() const { return new SimpleValue; }
	virtual ~SimpleValue();

	const char* getStringRepresentation() const
	{	return mValue->getStringRepresentation();	}
	const char* getStringContents() const
	{	return mValue->getStringContents();			}
	
	void setRestriction();

	//
	// --- ISerializable ---
	//

//	void		unmarshal(SAX::XMLReader<std::string> *parser, SAX::ContentHandler *parentHandler);
//	void		marshal(SAX::XMLReader<std::string> *parser, SAX::ContentHandler *parentHandler) {}
	/// get the stream representation of this meta information
	bool		unmarshal(std::istream &is, ISerializable *parent) { return true; }
	/// get the meta information from the stream representation
	bool		marshal(std::ostream& s, ISerializable *parent) { return true; }
	
protected :
	std::string mTypeName;
	Type mParentTypeId;
	IValue *mValue;
	Type mTypeId;

	Restriction *mRestriction;

	class SimpleValueHandler;
	friend class SimpleValueHandler;
	SimpleValueHandler *mHandler;
	
};

class Restriction
{
public:
	//RESTRICTIONs
	enum {
        LENGTH         = 1,
		MINLENGTH      = 1<<1,
		MAXLENGTH      = 1<<2,
		PATTERN        = 1<<3,
		ENUMERATION    = 1<<4,
		MAXINCLUSIVE   = 1<<5,
		MAXEXCLUSIVE   = 1<<6,
		MININCLUSIVE   = 1<<7,
		MINEXCLUSIVE   = 1<<8,
		TOTALDIGITS    = 1<<9,
		FRACTIONDIGITS = 1<<10,
		ENCODING       = 1<<11,
		DURATION       = 1<<12,
		PERIOD         = 1<<13,
		WHITESPACE     = 1<<14
    };

private:
};

#endif //__SIMPLEVALUE_H__
