// CHierNavHelperMM.h: interface for the CHierNavHelperMM class.
//
//////////////////////////////////////////////////////////////////////

#ifndef __CHIERNAVHELPERMM_H__
#define __CHIERNAVHELPERMM_H__

#include <xmdv/CHierNavHelper.h>

class CHierNavHelperMM : public CHierNavHelper  
{
public:

	CHierNavHelperMM();

	~CHierNavHelperMM();

	/// 
	/// A function that finds the number of levels in the cluster tree	
	/// 
	int FindTreeHeight();

	/// 
	/// A function that evaluates the level values and populates the array
	/// contour radius
	/// 
	/// @param countour_radius
	/// 
	void FindRadiusDistribution(ARR<double> &contour_radius);

	/// 
	/// A function that popultates the TreeContour array.
	/// That is evaluates the complete contour and popultate the arryy 
	/// tree_countour
	/// 
	/// @param tree_contour
	/// @param tree_colour
	/// 
	void FindTreeContour(ARR< ARR<double> > &tree_contour,
						 ARR< ARR<double> > &tree_colour,
						 ARR<double> contour_radius);

private:

	/// 
	/// A function that finds the tree height. This is a recursive function 
	/// that traverses the cluster_tree to to the deepest level to find the 
	/// height of the cluster tree.
	///
	void FindTreeHeight(Cluster* node,
						int curr_ht,
						int *max_ht);

	/// 
	/// A function that returns the and evaluates the tree contour. The 
	/// left and the right extent indicates what elements of the array are 
	/// to be set by the current call
	/// 
	/// @param node, a pointer to the root node
	/// @param curr_ht, gives the current level of depth
	/// @param rad, gives the level value that we are looking for using recursion
	/// @param contour, is one dimensional array that stores the value of the evaluated contour
	/// @param L, is the left extent
	/// @param R extent Right extent
	/// 
	void FindTreeContour(Cluster *node,
						int curr_ht,
						double rad,
						ARR<double> &contour,
						ARR<double> &color,
						double L,
						double R);

	/*
	*	Returns the radius of the root node.
	*/
	double CHierNavHelperMM::FindRootRadius();

};

#endif //__CHIERNAVHELPERMM_H__
