#ifndef CScreenDistortion_H
#define CScreenDistortion_H

#include <xmdv/ARR.h>

class CScreenDistortion
{
public:
    bool distortion_status;
    
    //control of on/off spacing 
    bool spacing;

    //parameters for parallel coordinate distortion
    ARR<double> paracoord_axes;
    ARR<double> paracoord_interval;

    //parameters for scatterplot matrix distortion
    ARR<double> scatter_X_axes;
    ARR<double> scatter_Y_axes;
    ARR<double> scatter_X_interval;
    ARR<double> scatter_Y_interval;

    //parameters for glyph distortion
    bool link;
    double threshold;
    int sort_dim;
    int link_method;
    
    ARR<double> star_angles;
    ARR<double> star_intervals;
    ARR<double> star_angles_cos_sized;
    ARR<double> star_angles_sin_sized;

    /*    ARR<double> mean_alpha;
    ARR<double> min_alpha;
    ARR<double> max_alpha;

    ARR<double> mean_sized_cos_alpha;
    ARR<double> mean_sized_sin_alpha;
    ARR<double> min_sized_cos_alpha;
    ARR<double> min_sized_sin_alpha;
    ARR<double> max_sized_cos_alpha;
    ARR<double> max_sized_sin_alpha;
*/    
    //parameters for scatterplot matrix distortion
    ARR<double> dimstack_size;
    double dimstack_X_blocksize;
    double dimstack_Y_blocksize;
    double dimstack_X_size;
    double dimstack_Y_size;

public:
	CScreenDistortion();
	~CScreenDistortion();
    void Reset();

    void ParacoordReset();
    void ParacoordResetWithSpacing(double *distances);
    void ParacoordScaleAxes(int dim, double delta);


    void ScatterReset();
    void ScatterScalePlot(int X_dim, int Y_dim, double alpha);

    void GlyphReset();
    void GlyphResetWithSpacing(double *distances);

    void DimstackReset();

    void InterRingReset();

protected:
    void ParacoordMapIntervalToAxes(ARR<double> &axes, ARR<double> &interval);
    void ParacoordScaleAxes(int dim, ARR<double> &axes, ARR<double> &interval, double delta);
    
    void ScatterScalePlotInOneDimension(int dim, ARR<double> &axes, ARR<double> &interval, double delta);
    void ScatterMapIntervalToAxes(ARR<double> &axes, ARR<double> &interval);

};

#endif
