///////////////////////////////////////////////////////////////////////////////
/// \file SettingsWindow.h
/// \brief Declaration of the CSettingsWindow class
/// \date 8/16/2004
/// \author anilkpatro
///////////////////////////////////////////////////////////////////////////////

#ifndef _SETTINGSWINDOW_H
#define _SETTINGSWINDOW_H

// SYSTEM INCLUDES
//
#include <string>

// PROJECT INCLUDES
//
#include <xmdv/XmdvTool.h>

#include <glgooey/WindowManager.h>
#include <glgooey/core/Rectangle.h>
#include <glgooey/FrameWindow.h>
#include <glgooey/CheckBox.h>
#include <glgooey/Button.h>
#include <glgooey/Panel.h>
#include <glgooey/EditField.h>
#include <glgooey/MultiTextButton.h>
#include <glgooey/StaticText.h>
#include <glgooey/ComboBox.h>
#include <glgooey/Slider.h>

// DEFINES
//

// FORWARD REFERENCES
//

///
/// Settings class for the Flat Pixel Oriented Display
///
/// USAGE:
/// <pre>
/// 	<Example usage of the class>
/// </pre>
/// \date 8/16/2004
/// \author anilkpatro
/// \version 1.0
///
class CFlatPixelSettings : public Gooey::FrameWindow
{
public:		// methods

// LIFECYCLE
//	
	///
	/// Default Constructor
	/// 
	CFlatPixelSettings(Tcl_Interp *interp);

	/// 
	/// Destructor
	/// 
	virtual	~CFlatPixelSettings();

// OPERATIONS
//
	void		Update(int w, int h);
	void		Reset();

	virtual void onShow();

	void		SetSelectedDim(int dim);
	void		SetSortedByDim(int dim);
	void		SetSortingOrder(bool asc);

protected:	// methods
	
// EVENT HANDLERS
//
	void		OnClicked_btnColors();
	void		OnClicked_btnWeights();
	void		OnClicked_btnBrushToolbox();
	void		OnClicked_btnBrushedVals();
	void		OnClicked_btnDataStats();

	void		OnClicked_chkShowColormap();
	void		OnClicked_chkShowDerived();
	void		OnClicked_chkMDSPlacement();
	void		OnClicked_chkSampling();

	void		OnSelChanged_cmbSortedBy(const std::string& newDim);
	void		OnSelChanged_cmbSortOrder(const std::string& newOrder);

	void		OnSelChanged_cmbSelectedDim(const std::string& newSelectedDim);
	void		OnValueChanged_sliDistortion(int value);
	void		OnValueChanged_sliSampling(int value);

private:	// members
	Tcl_Interp			*m_interp;

	Gooey::Panel		m_mainpanel;
	Gooey::Panel		m_horizpanel1;
	Gooey::Panel		m_horizpanel2;

	// the controls
	Gooey::Button		m_btnColors;
	Gooey::Button		m_btnWeights;
	Gooey::Button		m_btnBrushToolbox;
	Gooey::Button		m_btnBrushedVals;
	Gooey::Button		m_btnDataStats;

	Gooey::StaticText	m_lblSortedBy;
	Gooey::ComboBox		m_cmbSortedBy;
	Gooey::StaticText	m_lblOrder;
	Gooey::ComboBox		m_cmbSortOrder;

	Gooey::StaticText	m_lblSelect;
	Gooey::ComboBox		m_cmbSelectedDim;
	Gooey::Slider		m_sliDistortion;

	Gooey::CheckBox		m_chkShowColormap;
	Gooey::CheckBox		m_chkShowDerived;
	Gooey::CheckBox		m_chkMDSPlacement;
	Gooey::CheckBox		m_chkSampling;
	Gooey::Slider		m_sliSampling;
};

class PlusMinusPanel : public Gooey::Panel
{
public:
	
	PlusMinusPanel(const Gooey::Core::Rectangle &aRectangle,
				   Gooey::Window *window);

	void					OnShowInfo_Pressed();

private:
	
	// the controls
	Gooey::MultiTextButton	m_btnShowWindow;
	Gooey::Window			*m_theWindow;
};

// INLINE METHODS
//

#endif	// _SETTINGSWINDOW_H

