///////////////////////////////////////////////////////////////////////////////
/// \file PixelOriented.h
/// \brief Declaration of the CPixelOriented class
/// \date 7/12/2004
/// \author anilkpatro
///////////////////////////////////////////////////////////////////////////////

#ifndef _PIXELORIENTED_H
#define _PIXELORIENTED_H

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include <xmdv/ARR.h>
#include <xmdv/Vec2.h>

// DEFINES
//

// FORWARD REFERENCES
//
struct grCanvas;
class Brush;

/*
++
*/
struct QueryDist
{
	ARR<double> distances;
	ARR<int> indices;
	ARR<double> randomNums;
	double dist_min, dist_max;
	double brush_level;
	ARR<int> sizes;

	int data_size;
};

///
/// \ingroup 
///
/// <Brief Description>
///
/// USAGE:
/// <pre>
/// 	<Example usage of the class>
/// </pre>
/// \date 7/12/2004
/// \author anilkpatro
/// \version 1.0
/// \todo 
/// \bug 
///
class CPixelOriented
{
//Added by Zaixian start
public:
	enum DisplayType {
		PDT_SPIRAL,
		PDT_SEGMENTS,
	};

	enum PlacementType {
		PPT_NORMAL,
		PPT_MDS,
	};
//Zaixian end

protected:		// methods

  /// \name LIFECYCLE
  //@{
	
	///
	/// Constructor
	/// 
	CPixelOriented();

	/// 
	/// Copy constructor
	/// 
	/// \param from the value to copy to this object
	/// 
	CPixelOriented(const CPixelOriented& from);

	/// 
	/// Destructor
	/// 
	virtual ~CPixelOriented();

  //@}

  /// \name OPERATORS
  //@{

	/// 
	/// Assignment operator
	/// 
	/// \param from the value to assign to this object
	/// \return reference to this object
	CPixelOriented& operator = (const CPixelOriented& from);

  //@}

  /// \name OPERATIONS
  //@{

	virtual void		ComputeQueryDist(int sortedDim = 0, bool isAscending = true) = 0;

	virtual void		DrawAxesNameMinMax(grCanvas *canvas) =0;

	static double		YMAP(int X, double Y);
	static double		YMAP(double Y);
	static double		YINVMAP(int X, double Y);


	Vec2		MapScreenToCMapData(const Vec2& sc);
	static void	GetNormalizedBrushExtents(Brush* brush, int dim, 
										  double& max, double& min);

  //@}

  /// \name ACCESS
  //@{


	
  //@}
  
  /// \name INQUIRY
  //@{



  //@}

protected:	// members

	enum MarkerStatus {	
		MST_BLANK, 
		MST_ON_MIN, 
		MST_ON_MAX, 
		MST_DRAG_MIN, 
		MST_DRAG_MAX,
		MST_TRANSLATE,
		MST_RESIZE,
	};

	QueryDist		flat_query;
	QueryDist		hier_query;

//Added by Zaixian start
public:
	//Functions to setup the pixel-oriented display
	virtual void SetSortedDim(int dim) {};
	virtual void SetSortDimOrder(bool isAscending) {};
	virtual void SelectDim(int dim) {};
	virtual void SetDimDistortion(int dim, float dist) {};
	virtual void ShowColormap(bool bShow) {};
	virtual void ShowDerived(bool bShow) {};
	virtual void SetPlacementType(PlacementType pt) {};
	virtual void SetSamplingFactor(float val) {};
	virtual float GetDimDistortion(int dim) const {return 1.0;};
	virtual int GetSelectedDim() const {return 0;};
//Zaixian end
};

// INLINE METHODS
//

#endif	// _PIXELORIENTED_H
