////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  XmdvTool Version 4.0 - A program for Multivariate Data Display
//
//  Copyright 1994-1999, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty. 
//  In no event will the authors be held liable for any damages arising 
//  from the use of this software.
// 
//  Authors:
//      Matthew O. Ward, Allen R. Martin, Ying-Huey Fua
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~xmdv
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  XmdvTool Development Team: <matt@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //

#ifndef RGB_H
#define RGB_H

#include <iostream>

#define RValue(rgb)      ((unsigned char)(rgb))
#define GValue(rgb)      ((unsigned char)((rgb) >> 8))
#define BValue(rgb)      ((unsigned char)((rgb)>>16))

#define RGB2int(r,g,b) \
((unsigned int)(((unsigned char)(r)      | \
((unsigned int)((unsigned char)(g))<<8)) | \
((unsigned int)((unsigned char)(b))<<16)))

struct RGBt {
    double R,G,B,A;

	static const RGBt WHITE;
	static const RGBt BLACK;


    RGBt(void) : R(1.), G(1.), B(1.), A(1.) {}

    RGBt(double r,double g,double b, double a = 1.) 
		: R(r), G(g), B(b), A(a) {}

    RGBt(const unsigned long &c) {
        B = (c & 0xFF)/255.;
        G = ((c>>8) & 0xFF)/255.;
        R = ((c>>16) & 0xFF)/255.;
		A = ((c>>24) & 0xFF)/255.;
    }

	unsigned long GetColor() const {
		return ( ((unsigned long)( 255. * B))       |
				 ((unsigned long)( 255. * G)) << 8  |
				 ((unsigned long)( 255. * R)) << 16 |
				 ((unsigned long)( 255. * A)) << 24 );
	}

	void BlendTo(const RGBt& dest, double alpha)
	{
		R = R * alpha + ((1 - alpha)*dest.R),
		G = G * alpha + ((1 - alpha)*dest.G),
		B = B * alpha + ((1 - alpha)*dest.B);
	}

	friend RGBt Blend(const RGBt& src, const RGBt& dest, double alpha);
    friend std::ostream &operator<<(std::ostream &out, const RGBt &v);
};

inline std::ostream &operator<<(std::ostream &out, const RGBt &c)
{
    out << " [ r=" << c.R << " g=" << c.G << " b=" << c.B << " ] ";
    return out;
}

//shiping
inline std::istream & operator >> (std::istream &in, RGBt &v)
{
	in >>v.R>>v.G>>v.B;
	return in;
}
//shiping

// Anil [7/16/2004]
inline RGBt Blend(const RGBt& src, const RGBt& dest, double alpha)
{
	return RGBt(src.R * alpha + ((1 - alpha)*dest.R),
				src.G * alpha + ((1 - alpha)*dest.G),
				src.B * alpha + ((1 - alpha)*dest.B));
}
// Anil

#endif
