///////////////////////////////////////////////////////////////////////////
/// \file Subwindow.h
/// \brief Declaration of the Subwindow class
/// \date 5/6/2004
/// \author anilkpatro
///////////////////////////////////////////////////////////////////////////

#ifndef _SUBWINDOW_H
#define _SUBWINDOW_H

// SYSTEM INCLUDES
//

// PROJECT INCLUDES
//
#include <xmdv/ISubwindow.h>
#include <xmdv/Vec2.h>
#include <xmdv/CHierPixel.h>
#include <xmdv/BBox2D.h>

// DEFINES
//

// FORWARD REFERENCES
//
class Image;
class colormap;

///
/// \ingroup 
///
/// <Brief Description>
///
/// USAGE:
/// <pre>
/// 	<Example usage of the class>
/// </pre>
/// \date 5/6/2004
/// \author anilkpatro
/// \version 1.0
/// \todo 
/// \bug 
///
class Subwindow : public ISubwindow
{
public:		// methods

// LIFECYCLE
//
	///
	/// Constructor
	/// 
	Subwindow(int dim);

	/// 
	/// Copy constructor
	/// 
	/// \param from the value to copy to this object
	/// 
	Subwindow(const Subwindow& from);

	/// 
	/// Destructor
	/// 
	~Subwindow();

// OPERATORS
//
	/// 
	/// Assignment operator
	/// 
	/// \param from the value to assign to this object
	/// \return reference to this object
	Subwindow& operator = (const Subwindow& from);

// OPERATIONS
//
	void Paint(Image *img, const QueryDist& flat_query, 
			   double startSample = 0, double endSample = 1.0, 
			   bool hilite = false);
	void PaintClusters(Image *img, int size, const QueryDist& hier_query, 
					   CHierDisplayInformation *hier_info, 
					   bool hilite = false);

	void SetWindow(float x, float y, float width, float height);

// ACCESS
//
	BBox2D& GetBoundingBox();

	float GetX() const;
	void  SetX(float val);
	float GetY() const;
	void  SetY(float val);
	float GetWidth() const;
	float GetHeight() const;
	Vec2 GetCenter() const;
  
// INQUIRY
//
	bool Contains(float x, float y);
	bool Contains(float x, float y, int &n);

public:	// members

	int			m_dimension;
	BBox2D		m_box;
	int			m_size;

};

// INLINE METHODS
//

//-----------------------------------------------------------------------------
inline BBox2D& Subwindow::GetBoundingBox()
{
	return m_box;
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline Vec2 Subwindow::GetCenter() const
{
	return m_box.Center();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline float Subwindow::GetWidth() const
{
	return (float) m_box.Width();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline float Subwindow::GetHeight() const
{
	return (float) m_box.Height();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline float Subwindow::GetX() const
{
	return (float) m_box.Lowx();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline void Subwindow::SetX(float x)
{
	m_box.Shift(x, 0);
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline float Subwindow::GetY() const
{
	return (float) m_box.Lowy();
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline void Subwindow::SetY(float y)
{
	m_box.Shift(0, y);
}
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
inline bool Subwindow::Contains(float x, float y)
{
	return m_box.IsInside(x, y);
}
//-----------------------------------------------------------------------------

inline void Subwindow::SetWindow(float x, float y, float width, float height)
{
	m_box = BBox2D(x, y, x+width, y+height);
}

#endif	// _SUBWINDOW_H
