//modified by Jing Yang after 7/6/2001
#ifndef XMDVTOOL_H
#define XMDVTOOL_H

#include <map>

#include <xmdv/macros.h>
#include <xmdv/CDimReducer.h>
#include <xmdv/brush.h>
#include <xmdv/colorramp.h>
#include <xmdv/OrderDimension.h>

#define M_PI		3.14159265358979323846

/* SID */
extern int Prefetcher;
extern int BrushSemantic;		/* ALL = 0, ANY = 1, ... */
extern int Prefetch_Strategy;		/* */
extern bool backend;
extern int DSIZE;

enum interface_item { BG, TEXT, DATA, HIGHLIGHT1, HIGHLIGHT2, HIGHLIGHT3,
					  HIGHLIGHT4, OVERLAP, PAINT, GRID1, GRID2, GRID3,
					  GRID4, STAT1, STAT2, BRUSH1, BRUSH2, BRUSH3, BRUSH4,
					  SBB, NUM_INTERFACE_ITEM };
extern unsigned long interface_color[NUM_INTERFACE_ITEM];


// FORWARD REFERENCES
//
struct okcdata;
class CFlatDisplayInformation;
class CHierDisplayInformation;
class CDimReductionDisplayInformation;
class CScreenDistortion;
class CInterRingDimClusterTree;
class HistMgmt;

class CFlatParcoord;
class CFlatScatter;
class CFlatGlyph;
class CFlatDimstack;
class CFlatPixel;

class CHierParcoord;
class CHierScatter;
class CHierGlyph;
class CHierDimstack;
class CHierPixel;

class CHierNav;
class CHierHist;
class CParcoordTool;
class CHierTreemap;
class CGlyphTool;
class CDimstackTool;
class CPixelTool;
class CInterRingDimDisplay;
class CColorMapEditor;

struct grCanvas;

class CGlobals
{
public:
	CGlobals();
	~CGlobals();

	Brush brushes[MAXBRUSH];        // the brushes
	Brush all_brushes[MAXBRUSH];    // the complete brush set
	Brush *curr_brush;              // the current brush

	ClusterTree *btree;

	CFlatDisplayInformation			*flat_info;
	CHierDisplayInformation			*hier_info;
	CDimReductionDisplayInformation	*dim_reduction_info;
	CScreenDistortion				*screen_distortion;
	CInterRingDimClusterTree		*dtree;
	CDimReducer						*dim_reducer;
	HistMgmt						*histMgmt;	//YK

	char							*default_file_name;

	CFlatParcoord					*flatparcoord;
	CFlatScatter					*flatscatter;
	CFlatGlyph						*flatglyph;
	CFlatDimstack					*flatdimstack;
	CFlatPixel					*flatqipixel;

	CHierParcoord					*hierparcoord;
	CHierScatter					*hierscatter;
	CHierGlyph						*hierglyph;
	CHierDimstack					*hierdimstack;
	CHierPixel						*hierpixel;

	CHierNav						*hiernav;
	CHierHist						*hierhist;
	CParcoordTool					*parcoordtool;
	CHierTreemap					*hiertreemap;
	CGlyphTool						*glyphtool;
	CDimstackTool					*dimstacktool;
	CPixelTool						*pixeltool;
	CInterRingDimDisplay			*interring;
	CColorMapEditor					*cmapeditor;

	okcdata							*dataset;
	grCanvas						*mainCanvas;

	colormap						*pixelUnbrushedCMap;
	colormap						*pixelBrushedCMap;

	Reorder_Parameters				*reorderParameters;

	typedef std::map<std::string, colormap*> CMapRegistry;
	CMapRegistry					m_cmaps;

	double		GetRandomNum();
	void		RegisterCMap(colormap& cmap);
	void		UnregisterCMap(colormap& cmap);
	int buffersize;
};

extern CGlobals g_globals;

extern void appCanvasMousePan(struct grCanvas *);
extern void appRefreshAllCanvas(struct Tcl_Interp *interp,
								struct grCanvas *except_this = 0);
extern void appSetupBrushStatistics(struct Tcl_Interp *interp);
extern void appRefreshGlyphPlacementDialog(struct Tcl_Interp *interp);
extern void appUpdateClusterRadiusSliders(struct Tcl_Interp *interp);
extern void appUpdateTreeBrushEntries(struct Tcl_Interp *interp);
extern void appCanvasBrushHasChanged(Tcl_Interp *interp);
//extern void appShowCaption(Tcl_Interp *interp, const char *str);
extern void appShowCanvasMessage(Tcl_Interp *interp, const char *str,
								 const char *canvasName);
extern void appShowInterRingCaption(Tcl_Interp *interp, const char *str);
//it is used by brush tools. 
extern void appBrushBoundaryCaption(struct grCanvas *, int axis);
extern void appCanvasInputKeyPress(struct grCanvas *canvas, int key);
extern void appCanvasInputKeyRelease(struct grCanvas *canvas, int key);

//Anil
void appShowURIDialog(Tcl_Interp *interp, const char *fileName,
					  const char *fileContents);
//

//YS
enum ACTIONS {		//an A preceding a constant will mean it represents an "Action"
	A_EMPTY,		//to account for the array starting at 0
	A_HIER_L_B_H, 
	A_HIER_R_B_H, 
	A_HIER_I_C, 
	A_HIER_O_C, 
	A_EXIT,			//because input simulation uses 5 for exiting system
	A_HIER_M_B 
};
//YE

#endif
