////////////////////////////////////////////////////////////////// COPYRIGHT //
//
//  XmdvTool Version 4.0 - A program for Multivariate Data Display
//
//  Copyright 1994-1999, All Rights Reserved.
//
//  Permission to use, copy, modify and distribute this software and
//  its documentation for educational, research and non-profit
//  purposes, without fee, and without a written agreement is hereby
//  granted, provided that this copyright notice appear in all copies.
//
//  Permission to incorporate this software into commercial products
//  may be obtained directly from the authors.
//
//  This software is provided "as is" without express or implied warranty.
//  In no event will the authors be held liable for any damages arising
//  from the use of this software.
//
//  Authors:
//      Matthew O. Ward, Allen R. Martin, Ying-Huey Fua
//      Computer Science Department
//      Worcester Polytechnic Institute
//      Worcester, MA 01609 USA
//
//      http://davis.wpi.edu/~xmdv
//
//  Send bug reports, bug fixes, enhancements, requests, or comments to the
//  XmdvTool Development Team: <matt@cs.wpi.edu>.
//
////////////////////////////////////////////////////////////////// COPYRIGHT //

#ifndef BRUSH_H
#define BRUSH_H

#include <xmdv/ARR.h>

class Brush
{
    public:
        enum BoundaryType { STEP, RAMP };

        Brush() :
            enable(false),
            display(true),
            bound_type(STEP)
        {}

        bool IsPointCovered(const ARR<double> &point);
        void Resize(double factor);

		// PAUL: lang_opt brush_org
		// Nice inline functions
		inline double Pos(int d) const
			{	return pos[d];	}
        inline double Size(int d) const
			{	return size[d];	}
        inline double BoundOffset(int d) const
			{	return bound_offset[d];	}

		inline double Min(int d) const
			{	return min[d];	}
        inline double Max(int d) const
			{	return max[d];	}

        inline double RampMin(int d) const
			{	return ramp_min[d];	}
        inline double RampMax(int d) const
			{	return ramp_max[d];	}

		inline const ARR<double> & GetPos() const
			{	return pos;	}
		inline const ARR<double> & GetSize() const
			{	return size;	}
		inline const ARR<double> & GetBoundOffset() const
			{	return bound_offset;	}

		inline void SetLen(int dims)
		{
			pos.SetLen(dims);
			size.SetLen(dims);
			bound_offset.SetLen(dims);
			min.SetLen(dims);
			max.SetLen(dims);
			ramp_min.SetLen(dims);
			ramp_max.SetLen(dims);
		}

		void Init(int num, struct okcdata * dataset);

        void SetMinMax(int d,double lo,double hi);

		void SetPos(int d, double pos);
		void SetSize(int d, double size);
		void SetPosSize(int d, double pos, double size);
		void SetBoundOffset(int d, double offset);
		void SetTriple(int d, double pos, double size, double offset);
        
        //start_jing_animate_brush
        void Copy(Brush & brush);
        //end_jing_animate_brush

    public:
        int num;
        bool enable;              // if this brush is on
        bool display;             // display this brush?
        BoundaryType bound_type;       // brush boundary type

	protected:
		// Protecting data behind accessors makes it possible
		// to track the changes to brushes
        ARR<double> pos;
        ARR<double> size;
        ARR<double> bound_offset;  // boundary offset from min/max

		// PAUL: brush_org
		// Only compute these after a brush is changed
		ARR<double> min;
		ARR<double> max;
		ARR<double> ramp_min;
		ARR<double> ramp_max;

		friend class BrushMgr;
};

const int MAXBRUSH = 4;
//extern Brush brushes[MAXBRUSH], *curr_brush, all_brushes[MAXBRUSH];
extern unsigned long brush_color[MAXBRUSH];

extern void InitBrushSet();
extern int SetActiveBrush(int which);
extern int SetBrushEnableState(int which, bool state);
extern int SetBrushDisplayState(int which, bool state);
extern int SetBrushBoundaryType(int which, Brush::BoundaryType type);
extern bool CheckBrushCoverage(const ARR<double> &point, Brush *brush);
extern double PercentBrushCoverage(const ARR<double> &point, Brush *brush);
extern int CoverageRank(const ARR<double> &point);

#endif
