/* 
 Notes:
   The Link_List class is defined as a template class, which means 
   it's a generic class fit for all types of data, including user defined
   data type.

   The definitions used in doubly linked list are showed here:


   left_end          current                      right_end
      |                 |                            |
      |                 |                            |
     (0) <--> (1) <--> (2) <--> ... <--> (n-1) <--> (n)


   As you can see, the Link_List contains three important pointers, they 
   point to the left_end, current, and right_end respectively. Many of 
   the operations are concerned about the current pointer.

   (This is only the rough description of the link list, and does NOT
   trully represent the link list itself. Still much job is done
   concerning about the memory management and garbage collection, the 
   technical aspect of this program is listed in the end of this file.)

 --------------------------------------------------------------------------

 Example:
   Here list an example of how to use the Link_List class and how it works.

   First, include this file in your C++ file:
      #include "gu_link.h"

   If u had defined a class "Cat" before like this:
      class Cat
      {
      public:
         double weight;
          :
      }cat1, cat2, cat3;

   U have a lot of cats now and may want to link them togather,
   u can define a link list of class Cat like this:
      Link_List<Cat> cat_list;
      // --------------------------------
      //   left_end = current = right_end
      //                 |
      //              (NULL)
      // --------------------------------

   Next thing to do is to add cats to the list:
      cat_list.right_add(cat1); // add cat1 at right end.
      // -------------------------------
      //  left_end = current = right_end
      //                |
      // (NULL) <--> (cat1) <--> (NULL)
      // ------------------------------- 

      cat_list.right_add(cat2); // add cat2 at right end.
      // -------------------------------------------
      //   left_end = current     right_end
      //                  |           |
      //   (NULL) <--> (cat1) <--> (cat2) <--> (NULL)
      // ------------------------------------------- 
     
      cat_list.left_add(cat3); // add cat3 at left end.
      // -------------------------------------------------------
      //              left_end    current     right_end
      //                  |           |           |
      //   (NULL) <--> (cat3) <--> (cat1) <--> (cat2) <--> (NULL)
      // ------------------------------------------------------- 

      cat_list.at_left_end(); // put the current pointer at left_end.
      // -------------------------------------------------------
      //     left_end = current               right_end
      //                  |                       |
      //   (NULL) <--> (cat3) <--> (cat1) <--> (cat2) <--> (NULL)
      // -------------------------------------------------------

      cat_list++; // move the current pointer to the right one.
      // -------------------------------------------------------
      //              left_end    current    right_end
      //                  |           |           |
      //  (NULL) <--> (cat3) <--> (cat1) <--> (cat2) <--> (NULL)
      // ------------------------------------------------------- 

      cat_list.not_end();
      // -------------------------------------------------------
      //   if the current pointer pointer points at NULL, return 1,
      //   else return 0, in this case, 1 is returned.
      // ------------------------------------------------------- 

      cat_list.get();
      // -------------------------------------------------------
      //   this will return the element currently pointed, in this
      //   case, cat1 is returned.
      // ------------------------------------------------------- 

      cat_list[2];
      // -------------------------------------------------------
      //   this will return the 2nd element in the list, in this
      //   case, cat2 is returned, it seems like an array.
      // ------------------------------------------------------- 

      cat_list.left();
      // -------------------------------------------------------
      //   this will return the element left to the currently element,
      //   in this case, cat3 is returned.
      // ------------------------------------------------------- 

      cat_list.left_end();
      // -------------------------------------------------------
      //   this will return the element in left_end, in this
      //   case, cat2 is returned.
      // ------------------------------------------------------- 


      for( cat_list.at_left_end(); cat_list.not_end(); cat_list++ ) {
         cout << cat_list.get().weight << endl;
         cout << "one cat output." << endl;
      }
      // -------------------------------------------------------
      //   this for{} loop is a compound statement, it output the
      //   cats in list from left to right one by one, the list looks
      //   like this after the for{} loop:
      //
      //               left_end              right_end     current
      //                  |                       |           |
      //   (NULL) <--> (cat3) <--> (cat1) <--> (cat2) <--> (NULL)
      // ------------------------------------------------------- 

   Or u can do the above traverse in this way:
      for( int i=0; i<cat_list.count(); i++ ) {
         cout << cat_list[i].weight << endl;
         cout << "ont cat output." << endl;
      }

      cat_list--; cat_list--; // move current pointer left twice.
      // -------------------------------------------------------
      //               left_end    current    right_end
      //                  |           |           |
      //   (NULL) <--> (cat3) <--> (cat1) <--> (cat2) <--> (NULL)
      // ------------------------------------------------------- 

      cat_list.remove; // remove the current element from the list.
      // -------------------------------------------------------
      //               left_end    right_end
      //                  |           |
      //   (NULL) <--> (cat3) <--> (cat2) <--> (NULL)
      // ------------------------------------------------------- 

      Link_List<Cat> cat_list2; // define another list of Cat.
      cat_list2 = cat_list;
      // -------------------------------------------------------
      //    cat_list: (left_end    right_end)
      //                  |           |
      //   (NULL) <--> (cat3) <--> (cat2) <--> (NULL)
      //                  |           |
      //   cat_list2: (left_end    right_end)
      //
      // !!!!!!
      // the operator= is overloaded, when cat_list2 = cat_list,
      // they share the same link list. If one of them is changed,
      // the other is also changed.
      // ------------------------------------------------------- 

      //--------------------//
      //---- IMPORTANT  ----//
      //--------------------//
      cat_list.flush(); //Do flush() before a new born sub-linklist in 
                        //multiple-level link-list
      //--------------------------------------------------------
      //    (cat1) <--> (cat2) <--> (cat3)
      //       |                      |
      //    flush()                  flush()
      //   (cat11) <--> (cat12) ....   (cat31) <--> (cat32) ....
      //--------------------------------------------------------
   There are some other member functions of Link_List, such as:
   find(), sort(), flush(), deallocate()...
   U can see the comment of those functions to know how to use them.

 --------------------------------------------------------------------------

 Member functions:
		flush() --------- reset the list without freeing memory.
		deallocate() ---------- free the memory of this list.
		count() --------- return the element number of the list.
		find() ---------- to see if a specified element exists in the list.
		not_end() ------- to see if the current element is null or not.
		remove_current()- remove the current element.
		left_add() ------ add a new element at left end of the list.
		right_add() ----- add a new element at right end of the list.
		add_new() ------- add a new element which was nonexist in the list.
		at_left_end() --- place current pointer at left end of the list.
		at_right_end() -- place current pointer at right end of the list.
		get() ----------- return the element current pointed.
		left_end() ------ return the element at left end.
		right_end() ----- return the element at right end.
		append()--------- add another linked list to the current one.
		operator[i] ----- return the i th element in the list(from left).
		operator++(int) - return the current element and move to right.
		operator--(int) - return the current element and move to left.
		sort() ---------- sort the list by smallest at left end.
      		operator= ------- overload to allow reference count.

 --------------------------------------------------------------------------
                      Mission one start...
                          Good luck !
 ----------------------------------------------------------------------- */


#if !defined(GU_DOUBLY_LINKED_LIST_H)
#define GU_DOUBLY_LINKED_LIST_H

//#define GU_LIST_DEBUG  // uncomment this line to allow debug infomation.

// ------------------
#include <iostream>
#include <fstream>
#include <cstdio>
#include <cstdlib>
#include "gu.h"

using namespace std;

#define list_traverse(a) a.at_left_end(); a.not_end(); a++

// ------------------ generic list object class ---------------------
// define the template class to hold the user defined object.
// ------------------------------------------------------------------
//##ModelId=3ADAA3630021
template <class holder_type> 
class List_Object
{
	public:
	//##ModelId=3ADAA3630049
		holder_type element;
	//##ModelId=3ADAA363003F
		List_Object<holder_type> *left;
	//##ModelId=3ADAA363002F
		List_Object<holder_type> *right;
	//##ModelId=3ADAA363004A
		List_Object()           { left = right = NULL; }
};

/////////////////////////////////////////////////////////////////////
// the class implementation of generic class of doubly linked list  /
//                                                                  /
/////////////////////////////////////////////////////////////////////
//##ModelId=3ADAA36701C1
template <class holder_type> 
class Link_List 
{
public:
	void flush();
	void deallocate();
	int count() const;
	int find(const holder_type&) const;
	int  not_end() const;
	void remove_current();
	//added by sqy
	void remove_current(int index);

	void left_add(const holder_type&);
	void right_add(const holder_type&);
	void add_new(const holder_type&);
	//##ModelId=3ADAA36701EC
	void at_left_end()  const  { p_current() = p_left_end();  }
	//##ModelId=3ADAA36701EB
	void at_right_end() const  { p_current() = p_right_end(); }
	holder_type& get();
	holder_type  get() const;
	holder_type& left();
	holder_type  left() const;
	holder_type& right();
	holder_type  right() const;
	holder_type& left_end();
	holder_type  left_end() const;
	holder_type& right_end();
	holder_type  right_end() const;
	void append(const Link_List<holder_type>&);
	holder_type& operator[](int);
	holder_type  operator[](int) const;
	void operator++(int) const;
	void operator--(int) const;
	void copy_from(const Link_List<holder_type>&);
	void sort();
	void ascend_insert(const holder_type&);

	Link_List(); 
	Link_List(const Link_List<holder_type>&);
	//##ModelId=3ADAA36701EA
	~Link_List() { decrease(); }
	Link_List<holder_type>& operator=(const Link_List<holder_type>&);
	int operator==(const Link_List<holder_type>&);

	// --- IO functions ---
	friend std::ostream& operator<<(std::ostream& s, const Link_List<holder_type>& a)
	{
	//revised by sqy, June 21,2001. for the output for partfamily object
	//	s << " list_size: " << a.count() << endl;
		for( list_traverse(a) )  s << a.get() << endl;
	//	s << endl;
		return s;
	}
	
	friend std::ofstream& operator<<(std::ofstream& s, const Link_List<holder_type>& a)
	{
	//	s << " list_size: " << a.count() << endl;
		for( list_traverse(a) )  s << a.get() << endl;
	//	s << endl;
		return s;
	}
	
	friend std::istream& operator>>(std::istream& s, Link_List<holder_type>& a)
	{
		int size;
		char junk[20];
		holder_type b;
	
		s >> junk >> size;
		for( int i=0; i<size; i++ ) {
			s >> b;
			a.right_add(b);
		}
		return s;
	}
	
	friend std::ifstream& operator>>(std::ifstream& s, Link_List<holder_type>& a)
	{
		int size;
		char junk[20];
		holder_type b;
	
		s >> junk >> size;
		for( int i=0; i<size; i++ ) {
			s >> b;
			a.right_add(b);
		}
		return s;
	}
	
	//##ModelId=3ADAA36701CB
//	friend std::ostream& operator<<(std::ostream&, const Link_List<holder_type>&);
	//##ModelId=3ADAA36701CE
//	friend std::ofstream& operator<<(std::ofstream&, const Link_List<holder_type>&);
	//##ModelId=3ADAA36701D1
//	friend std::istream& operator>>(std::istream&, Link_List<holder_type>&);
	//##ModelId=3ADAA36701D7
//	friend std::ifstream& operator>>(std::ifstream&, Link_List<holder_type>&);

private:
	#if defined(GU_LIST_DEBUG)
		static int total_list_number;
	#endif
	//##ModelId=3ADAA36701DB
	int *refer_count;
	List_Object<holder_type> **pp_current, **pp_left_end, **pp_right_end;
	//##ModelId=3ADAA36701DA
	holder_type garbage;

	//##ModelId=3ADAA36701E9
	List_Object<holder_type>*& p_current() const { return *pp_current; }
	//##ModelId=3ADAA36701E1
	List_Object<holder_type>*& p_left_end() const { return *pp_left_end; }
	//##ModelId=3ADAA36701E0
	List_Object<holder_type>*& p_right_end() const { return *pp_right_end; }
	//##ModelId=3ADAA36701DF
	void increase() { ++(*refer_count); }
	//##ModelId=3ADAA36701DC
	void decrease() { if( --(*refer_count)==0 ) die_hard(); }
	void new_born();
	void die_hard();
	void disconnect();
	void connect_to(const Link_List<holder_type>& list_in);
};

#if defined(GU_LIST_DEBUG)
	template <class holder_type> 
	int Link_List<holder_type>::total_list_number = 0;
#endif


// ---------------- Link_List::constructor --------------------------
template <class holder_type> 
inline Link_List<holder_type>::Link_List()
{ 
	new_born();
}

// ---------------- Link_List::copy constructor ---------------------
template <class holder_type> 
inline Link_List<holder_type>::Link_List(const Link_List<holder_type>& list_in)
{
	connect_to(list_in);
}

// ---------------- Link_List::flush() ------------------------------
// Reset the link list to empty without freeing the memory.
// ------------------------------------------------------------------
template <class holder_type> 
inline void Link_List<holder_type>::flush()
{
	disconnect();
	new_born();
}

// ---------------- Link_List::left_end() ---------------------------
// return the left end element.
// ------------------------------------------------------------------
template <class holder_type> 
inline holder_type& Link_List<holder_type>::left_end()
{
	if( p_left_end() == NULL )  return garbage;
	return p_left_end()->element;
}

template <class holder_type> 
inline holder_type Link_List<holder_type>::left_end() const
{
	if( p_left_end() == NULL )  return garbage;
	return p_left_end()->element;
}

template <class holder_type> 
inline holder_type& Link_List<holder_type>::right_end()
{
	if( p_left_end() == NULL )  return garbage;
	return p_right_end()->element;
}

template <class holder_type> 
inline holder_type Link_List<holder_type>::right_end() const
{
	if( p_left_end() == NULL )  return garbage;
	return p_right_end()->element;
}


// ---------------- Link_List::deallocate() -------------------------------
// Free the memory of this link list.
// ------------------------------------------------------------------
template <class holder_type> 
inline void Link_List<holder_type>::deallocate()
{
	List_Object<holder_type> *p, *p1;
	
	// free all elements in the list
	p = p_left_end();
	while(p) {
		p1 = p->right;  // if it had already been freed,
		delete p;      // then ignore the free action.
		p = p1;
	}
	p_current() = p_left_end() = p_right_end() = NULL;
}

// ---------------- Link_List::count() ------------------------------
// Free the memory of this link list.
// ------------------------------------------------------------------
template <class holder_type> 
inline int Link_List<holder_type>::count() const
{
	if( p_left_end() == NULL ) return 0;
	
	int element_number = 0;
	List_Object<holder_type> *p;
	p = p_left_end();
	while( p ) {
		p = p->right;
		element_number++;
	}
	return element_number;
}

// ------------- Link_List::not_end() -------------------------------
// Tell if the current position is at the end of link list or not.
// ------------------------------------------------------------------
template <class holder_type> 
inline int Link_List<holder_type>::not_end() const
{
	return !(p_current() == NULL);
} 

// ------------- Link_List::right_add() -----------------------------
// add the element to the right end of the link list.
// ------------------------------------------------------------------
template <class holder_type> 
inline void Link_List<holder_type>::right_add(const holder_type& add_me)
{
	List_Object<holder_type> *p;

	gu_new_single(p);

	p->element = add_me;
	if ( p_left_end() == NULL ) 	//first element in list
		p_current() = p_left_end() = p_right_end() = p;
	else { 				//put on right_end
		p->left = p_right_end();
		p_right_end()->right = p;
		p_right_end() = p;
	}
}

// ------------- Link_List::left_add() ------------------------------
// add the element to the left end of the link list.
// ------------------------------------------------------------------
template <class holder_type> 
inline void Link_List<holder_type>::left_add(const holder_type& add_me)
{
	List_Object<holder_type> *p;

	gu_new_single(p);

	p->element = add_me;
	if ( p_left_end() == NULL ) 	//first element in list
		p_current() = p_left_end() = p_right_end() = p;
	else { 					//put on left_end
		p->right = p_left_end();
		p_left_end()->left = p;
		p_left_end() = p;
	}
}

// ------------- Link_List::add_new() ------------------------------
// add the new element which has not been found in current linklist.
// ------------------------------------------------------------------
template <class holder_type> 
inline void Link_List<holder_type>::add_new(const holder_type& add_me)
{
	for(at_left_end();not_end();(*this)++)
	{
	    if(get() == add_me) return;
	}
	right_add(add_me);
}

// ---------------- Link_List::get() --------------------------------
// return the element at current position.
// ------------------------------------------------------------------
template <class holder_type> 
inline holder_type& Link_List<holder_type>::get()
{
	if ( p_current() == NULL ) {
		cerr << "!Link_List::get:try to get from null." << endl;
		return garbage;
	}
	return p_current()->element;
}

// ---------------- Link_List::get() const --------------------------
// return the element at current position.
// ------------------------------------------------------------------
template <class holder_type> 
inline holder_type Link_List<holder_type>::get() const
{
	if ( p_current() == NULL ) {
		cerr << "!Link_List::get:try to get from null." << endl;
		return garbage;
	}
	return p_current()->element;
}

// ---------------- Link_List::left() ---------------------------
// return the element at left position.
// ------------------------------------------------------------------
template <class holder_type> 
inline holder_type& Link_List<holder_type>::left()
{
	if ( p_current()->left == NULL ) {
		cerr << "!Link_List::get_left:try to get from null." << endl;
		return garbage;
	}
	return p_current()->left->element;
}

// ---------------- Link_List::left() const -------------------------
// return the element at left position.
// ------------------------------------------------------------------
template <class holder_type> 
inline holder_type Link_List<holder_type>::left() const
{
	if ( p_current()->left == NULL ) {
		cerr << "!Link_List::get_left:try to get from null." << endl;
		return garbage;
	}
	return p_current()->left->element;
}

// ---------------- Link_List::right() --------------------------
// return the element at right position.
// ------------------------------------------------------------------
template <class holder_type> 
inline holder_type& Link_List<holder_type>::right()
{
	if ( p_current()->right == NULL ) {
		cerr << "!Link_List::get_right:try to get from null." << endl;
		return garbage;
	}
	return p_current()->right->element;
}

// ---------------- Link_List::right() const ------------------------
// return the element at right position.
// ------------------------------------------------------------------
template <class holder_type> 
inline holder_type Link_List<holder_type>::right() const
{
	if ( p_current()->right == NULL ) {
		cerr << "!Link_List::get_right:try to get from null." << endl;
		return garbage;
	}
	return p_current()->right->element;
}

// --------------- Link_List::append --------------------------------
// combine two linked list togather
// ------------------------------------------------------------------
template<class holder_type>
inline void Link_List<holder_type>::append(const Link_List<holder_type>& ll)
{
	for(list_traverse(ll)) right_add(ll.get());
}
// --------------- Link_List::operator[] ----------------------------
// this can be used to simulate an dynamic array!
// ------------------------------------------------------------------
template <class holder_type> 
inline holder_type& Link_List<holder_type>::operator[](int index)
{
	if( p_left_end() == NULL ) {
		std::cerr << "!Link_List::operator[]:try to get from null." << std::endl;
		return garbage;
	}
	if( index<0 || index>=count() ) {
		std::cerr << "!Link_List::operator[]:invalid index number." << std::endl;
		return garbage;
	}
	List_Object<holder_type> *p = p_left_end();
	for( int i=0; i<index; i++ ) p = p->right;
	return p->element;
}

// --------------- Link_List::operator[] const ----------------------
// this can be used to simulate an dynamic array!
// ------------------------------------------------------------------
template <class holder_type> 
inline holder_type Link_List<holder_type>::operator[](int index) const
{
	if( p_left_end() == NULL ) {
		cerr << "!Link_List::operator[]:try to get from null." << endl;
		return garbage;
	}
	if( index<0 || index>=count() ) {
		cerr << "!Link_List::operator[]:invalid index number." << endl;
		return garbage;
	}
	List_Object<holder_type> *p = p_left_end();
	for( int i=0; i<index; i++ ) p = p->right;
	return p->element;
}

// --------------- Link_List::postfix ++ ----------------------------
// move to right element.
// ------------------------------------------------------------------
template <class holder_type>
inline void Link_List<holder_type>::operator++(int) const
{
	if ( p_current() != NULL ) {
		p_current() = p_current()->right;
		return;
	}

	cerr << "!link list::operator++:reached end." << endl;
}

// --------------- Link_List::postfix -- ----------------------------
// move to left element.
// ------------------------------------------------------------------
template <class holder_type>
inline void Link_List<holder_type>::operator--(int) const
{
	if ( p_current() != NULL ) {
		p_current() = p_current()->left;
		return;
	}

	cerr << "!link list::operator--:reached end." << endl;
}       

// --------------- Link_List::find() -------------------------------- 
// to check if a given element is in the link list or not.
// if found, the index number is returned, else return -1.
// so check the returned value before further operations.
// if the element is not a build in type, the == should be defined.
// ------------------------------------------------------------------
template <class holder_type>
inline int Link_List<holder_type>::find(const holder_type& find_me) const
{
	if( p_left_end() == NULL ) return -1; // null list.
	List_Object<holder_type> *p;

	p = p_left_end();
	int index = 0;
	while (p != NULL) {
		if (p->element == find_me) 
			return index; // found it.
		else {
			p = p->right; index++;
		}
	}
	return -1;     // not found.
}

// --------------- Link_List::remove_current() ----------------------
// remove current element from the link list.
// after every remove the current() is at left end.
// ------------------------------------------------------------------
template <class holder_type>
inline void Link_List<holder_type>::remove_current()
{
	if( p_left_end() == NULL ) return;

	List_Object<holder_type> *p;
	p = p_current();

	if( p == NULL ) {
		cout << "!Link_List::remove_current:null." << endl;
		return;
	}
	// -1- remove the only one in list ---
	if ( (p==p_left_end()) && (p_left_end()==p_right_end()) ) {
		p_left_end() = p_right_end() = NULL;
		delete p; 
		p_current() = p_left_end();
		return;
	}
	// -2- remove the left end ---
	if (p == p_left_end()) {
		p_left_end() = p_left_end()->right;
		p_left_end()->left = NULL;
		delete p; 
		p_current() = p_left_end();
		return;
	}
	// -3- remove the right end ---
	if (p == p_right_end()) {
		p_right_end() = p_right_end()->left;
		p_right_end()->right = NULL;
		delete p; 
		p_current() = p_left_end();
		return;
	}
	// -4- normal situation ---
	p_current() = p_current()->left;
	p->left->right = p->right;
	p->right->left = p->left;
	delete p; p = NULL;
	p_current() = p_left_end();
	return;
}

// --------------- Link_List::remove_current(index) ----------------------
// remove current element from the link list.
// after every remove the current() is at left end.
// ------------------------------------------------------------------
template <class holder_type>
inline void Link_List<holder_type>::remove_current(int index)
{
	List_Object<holder_type> *p;
	
	if( p_left_end() == NULL ) return;
	
	//added by sqy, 6.25.2001

	//for the node which is selected to remove
	
	p = p_left_end();
	
	if(index >0){
		for(int i =0 ; i < index ;i++)
			p = p -> right;
	}
	
	//p_current() =  

	

	//p = p_current();

	if( p == NULL ) {
		cout << "!Link_List::remove_current:null." << endl;
		return;
	}
	// -1- remove the only one in list ---
	if ( (p==p_left_end()) && (p_left_end()==p_right_end()) ) {
		p_left_end() = p_right_end() = NULL;
		delete p; 
		p_current() = p_left_end();
		return;
	}
	// -2- remove the left end ---
	if (p == p_left_end()) {
		p_left_end() = p_left_end()->right;
		p_left_end()->left = NULL;
		delete p; 
		p_current() = p_left_end();
		return;
	}
	// -3- remove the right end ---
	if (p == p_right_end()) {
		p_right_end() = p_right_end()->left;
		p_right_end()->right = NULL;
		delete p; 
		p_current() = p_left_end();
		return;
	}
	// -4- normal situation ---
	p_current() = p_current()->left;
	p->left->right = p->right;
	p->right->left = p->left;
	delete p; p = NULL;
	p_current() = p_left_end();
	return;
}


// ----------------- Link_List::ascend_insert() -------------------
// insert a new element into the list, right before the element
// that is larger than it.
// ----------------------------------------------------------------
template <class holder_type>
inline void Link_List<holder_type>::ascend_insert(const holder_type& a)
{
	List_Object<holder_type> *pa;
	gu_new_single(pa);
	pa->element = a;

	List_Object<holder_type> *p=p_left_end();
	while( p ) {
		if( a < p->element ) {
			if( p == p_left_end() ) {
				left_add(a);
				return;
			}
			pa->left = p->left;
			pa->right = p;
			pa->left->right = pa;
			pa->right->left = pa;
			return;
		}
		p = p->right;
	}
	// --- if empty list or largest element ---
	right_add(a);
	return;
}

// ----------------- Link_List::copy_from() -------------------------
// copy the content of another link list.
// ------------------------------------------------------------------
template <class holder_type>
void Link_List<holder_type>::copy_from(const Link_List<holder_type>& copy_me)
{
	flush();
	List_Object<holder_type> *p;
	p = copy_me.p_left_end();

	while( p ) {
		right_add(p->element);
		p = p->right;
	}	
}

// ----------------- Link_List::sort() ----------------------------
// bubbling sort, the < operator of list object should be defined.
// the largest element is at the left end after sorting.
// ----------------------------------------------------------------
template <class holder_type>
inline void Link_List<holder_type>::sort()
{
	if( count() < 2 ) return;

	holder_type temp;
	List_Object<holder_type> *p, *p_right;

	int done = 0;	
	while(!done) {
		done = 1;
		p = p_left_end();
		while( p && p->right) {
			p_right=p->right;
			if( p_right->element < p->element ) {
				swap( p->element, p_right->element);
				done = 0;
			}
			p = p->right;
		}
	}
}
/*
template <class holder_type>
std::ostream& operator<<(std::ostream& s, const Link_List<holder_type>& a)
{
//revised by sqy, June 21,2001. for the output for partfamily object
//	s << " list_size: " << a.count() << endl;
	for( list_traverse(a) )  s << a.get() << endl;
//	s << endl;
	return s;
}

template <class holder_type>
std::ofstream& operator<<(std::ofstream& s, const Link_List<holder_type>& a)
{
//	s << " list_size: " << a.count() << endl;
	for( list_traverse(a) )  s << a.get() << endl;
//	s << endl;
	return s;
}

template <class holder_type>
std::istream& operator>>(std::istream& s, Link_List<holder_type>& a)
{
	int size;
	char junk[20];
	holder_type b;

	s >> junk >> size;
	for( int i=0; i<size; i++ ) {
		s >> b;
		a.right_add(b);
	}
	return s;
}

template <class holder_type>
std::ifstream& operator>>(std::ifstream& s, Link_List<holder_type>& a)
{
	int size;
	char junk[20];
	holder_type b;

	s >> junk >> size;
	for( int i=0; i<size; i++ ) {
		s >> b;
		a.right_add(b);
	}
	return s;
}
*/

/* ---------------- Link_List::operator= --------------------------
 If B = A:

  +-------------------------------------------------------------+
  | B |    pp_left_end -+   pp_current -+    pp_right_end -+    |
  +---------------------|---------------|------------------|----+
                        |               |                  |
  +---------------------|---------------|------------------|----+
  | A |    pp_left_end  |   pp_current  |    pp_right_end  |    |
  +---------------------|---------------|------------------|----+
               |________|        |______|          |_______|
              \|/               \|/               \|/
  +-------------------------------------------------------------+
  |         p_left_end       p_current        p_right_end       |
  |            |                 |                 |            |        
  |           \|/               \|/               \|/           |
  | NULL <--> (*) <--- ... ---> (*) <--- ... ---> (*) <--> NULL |
  +-------------------------------------------------------------+

B & A share the same memory.
------------------------------------------------------------------ */
template <class holder_type> 
inline Link_List<holder_type>& Link_List<holder_type>::operator=(const Link_List<holder_type>& list_in)
{
	if( this == &list_in ) return *this;
	disconnect();
	connect_to(list_in);
	return *this;
}

template <class holder_type> 
inline int Link_List<holder_type>::operator==(const Link_List<holder_type>& list_in)
{
	int flag;
	int n = this->count();

	flag = 0;
	if( this == &list_in ) return 1;
	if( list_in.count() != n) return 0;
	for(int i =0; i<n; i++)
	{
	    if((*this)[i] == list_in[i])
		flag++;
	}
	if(flag == n)
	    return 1;
	else
	    return 0;
}


/* ----------------- Link_List::new_born() --------------------------
 Before new_born():
        pp_left_end      pp_current     pp_right_end
            |________________ | ______________|                
                             \|/
                            (NULL)

 After new_born():
        pp_left_end      pp_current     pp_right_end
            |                 |               |
           \|/               \|/             \|/
      +-----------------------------------------------+
      |  p_left_end       p_current      p_right_end  |
      |     |________________ | ______________|       |        
      |                      \|/                      |
      |                     (NULL)                    |
      +-----------------------------------------------+
------------------------------------------------------------------ */
template <class holder_type> 
inline void Link_List<holder_type>::new_born()
{
	gu_new_single(refer_count);  (*refer_count) = 1;
	gu_new_single(pp_current);   p_current() = NULL;
	gu_new_single(pp_left_end);  p_left_end() = NULL;
	gu_new_single(pp_right_end); p_right_end() = NULL;
	#if defined(GU_LIST_DEBUG)
		total_list_number++;
		cout << "list number: " << total_list_number << endl;
	#endif
}

/* ----------------- Link_List::die_hard() --------------------------
 Before die_hard():
           pp_left_end      pp_current       pp_right_end
               |                 |                 |
              \|/               \|/               \|/
  +-------------------------------------------------------------+
  |         p_left_end       p_current        p_right_end       |
  |            |                 |                 |            |        
  |           \|/               \|/               \|/           |
  | NULL <--> (*) <--- ... ---> (*) <--- ... ---> (*) <--> NULL |
  +-------------------------------------------------------------+

 After die_hard():
        pp_left_end      pp_current     pp_right_end
            |________________ | ______________|                
                             \|/
                            (NULL)

 The link list is destroyed.
------------------------------------------------------------------ */
template <class holder_type> 
inline void Link_List<holder_type>::die_hard()
{
	deallocate(); 
	delete refer_count;  refer_count = NULL;
	delete pp_current;   pp_current = NULL;
	delete pp_left_end;  pp_left_end = NULL;
	delete pp_right_end; pp_right_end = NULL;
	#if defined(GU_LIST_DEBUG)
		total_list_number--;
		cout << "list number: " << total_list_number << endl;
	#endif
}

/* ----------------- Link_List::disconnect() --------------------------
 Before disconnect():
           pp_left_end      pp_current       pp_right_end
               |                 |                 |
              \|/               \|/               \|/
  +-------------------------------------------------------------+
  |         p_left_end       p_current        p_right_end       |
  |            |                 |                 |            |        
  |           \|/               \|/               \|/           |
  | NULL <--> (*) <--- ... ---> (*) <--- ... ---> (*) <--> NULL |
  +-------------------------------------------------------------+

 After disconnect():
        pp_left_end      pp_current     pp_right_end
            |________________ | ______________|                
                             \|/
                            (NULL)

 The link list still remain in the memory, it's still in use by 
 other Link_List object.
------------------------------------------------------------------ */
template <class holder_type> 
inline void Link_List<holder_type>::disconnect()
{
	decrease();
	refer_count = NULL;
	pp_current = pp_left_end = pp_right_end = NULL;
}

/* ----------------- Link_List::connect_to() --------------------------
 Before connect_to():
        pp_left_end      pp_current     pp_right_end
            |________________ | ______________|                
                             \|/
                            (NULL)

 After connect_to():
           pp_left_end      pp_current       pp_right_end
               |                 |                 |
              \|/               \|/               \|/
  +-------------------------------------------------------------+
  |         p_left_end       p_current        p_right_end       |
  |            |                 |                 |            |        
  |           \|/               \|/               \|/           |
  | NULL <--> (*) <--- ... ---> (*) <--- ... ---> (*) <--> NULL |
  +-------------------------------------------------------------+

------------------------------------------------------------------ */
template <class holder_type> 
inline void Link_List<holder_type>::connect_to(const Link_List<holder_type>& list_in)
{
	refer_count = list_in.refer_count;
	pp_current = list_in.pp_current;
	pp_left_end = list_in.pp_left_end;
	pp_right_end = list_in.pp_right_end;
	increase();
}


/* ------------------------------------------------------------------

 The core part of this program is concerning about the garbage collection,
 i.e., how to recycle the memory used by the link list. Because of the 
 nature of link list, it's too consuming to copy the whole list. So, only
 the reference is used in list asignment.(i.e., if list B = A, then A & B
 point to the same memory, see the figure of operator=.)

 A problem come with this memory management method is:
 When should the memory be freed? How to free it?
 
 One solution is to free the memory in object's destructor, but the 
 problems are:

 If two list share the same memory, one of them die first and freed the
 shared the memory, what should the other list do? And when the second
 dies, it will try to free the memory already freed, which will cause
 GP error.

 So, one possible solution to the problem is, try to find out who is the
 last one points to the list, and he is resposible to free the memory.
 But how to know who's the last one? Here comes the answer:

 "Reference Count".

 Reference count means each list has a counter, which keeps the number
 of Link_List objects pointing to him. When the counter decrease to 0,
 it will call the function to free the memory. 

 In the program, the reference count is done by:
 one private data member: int refer_count;
 two private member functions: decrease() & increase().


 But why use pp_left_end in the program?

 Let check this situation: Two list object points to the same memory,
 one of them added a new element to the list and updated it's left_end
 or right_end pointer, but the other one had no way to know about that,
 it's left_end or right_end pointer still points to the old position,
 and then ... error occurs ... crack!

 So, there have to be a way to keep all the list objects updated, that's
 why the pointer's pointer is used here. p_left_end points to the left
 end element, and pp_left_end points to p_left_end.

------------------------------------------------------------------ */



#endif
