#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <fstream>
#include <cctype>

#include <IP_Array.h>
#include <IP_Vector.h>
#include <IP_Matrix.h>


struct Tex_Unit
{
	char word[256];
	int count;
	friend std::ostream& operator <<(std::ostream& os, Tex_Unit tu) 
	{  //<<tu.word<<'\t'<<tu.count<<endl;
		os<<tu.count;
		return os;
	}
	friend std::istream& operator >>(std::istream& is, Tex_Unit tu) 
	{  is>>tu.word>>tu.count;
		return is;
	}

	int operator > (const Tex_Unit& tu) const {return (count>tu.count?1:0);}
	int operator < (const Tex_Unit& tu) const {return (count<tu.count?1:0);}

};

/*
 * change the rainbow aiw matrix output to okc file
 * argv[1]: rainbow matrix output (--print-matrix=aiw)
 * argv[2]: okc file name
 *
 */

int main (int argc, char *argv[]) 
{
	if (argc<3) {
		std::cout<<"Required parameters:"<<std::endl;
		std::cout<<"  argv[1]: rainbow matrix output (--print-matrix=aiw)"<<std::endl;
		std::cout<<"  argv[2]: okc file name"<<std::endl;
		return 1;
	}

	std::ifstream inf;
	inf.open (argv[1]);
	if (inf==NULL) {
		std::cout<<"cannot open input file!!"<<std::endl;
		return 1;
	}

	/*
	 * get the number of rows (n) and colums (m)
	 * n: number of data records
	 * m: number of dimensions
	 */
	char tt[512];
	int n=0; 
	int m=0;
	while (!inf.eof ()) {
		n++;
		inf>>tt>>tt;

		Tex_Unit tu;
		m=0;
		while (inf.peek () !='\n') {
			m++;
			
			inf>>tu;
			while (inf.peek() ==' ' ||inf.peek() == '\t')inf.get ();
		}
		// TODO: Find substitute for eatwhite
//		inf.eatwhite ();
	}
	//cout<<"n="<<n<<'\t'<<"m="<<m<<endl;
	inf.close ();

	
	// get the maximum and minimum values for colums (dimensions)
	inf.open (argv[1]);
	if (inf==NULL) {
		std::cout<<"cannot open input file!!"<<std::endl;
		return 1;
	}

	IP_Vector<Tex_Unit> row(m);
	IP_Vector<int> colmin(m), colmax(m);
	for (int j=0; j<m; j++) colmin(j)=colmax(j)=0;
	for (int i=0; i<n; i++) {
		inf>>tt>>tt;
		inf>>row;
		for (int j=0; j<m; j++) {
			if (row(j).count>colmax(j)) colmax(j)=row(j).count;
			if (row(j).count<colmin(j)) colmin(j)=row(j).count;
		}
	}
	inf.close();


	// output the .okc file
	std::ofstream of;
	of.open (argv[2]);
	if (of==NULL) {
		std::cout<<"cannnot open output file!!"<<std::endl;
		return 1;
	}
	char tbl='\t';
	of<<m<<tbl<<n<<std::endl;
	for ( j=0; j<m; j++) 
		of<<row(j).word<<std::endl;
	for (j=0; j<m; j++)
		of<<colmin(j)<<tbl<<colmax(j)<<tbl<<'1'<<std::endl;

	inf.open (argv[1]);
	if (inf==NULL) {
		std::cout<<"cannot open input file!!"<<std::endl;
		return 1;
	}
	for ( i=0; i<n; i++) {
		inf>>tt>>tt;
		inf>>row;
		of<<row<<std::endl;
	}
	inf.close();
	of.close ();

	std::cout<<"Your OKC file is ready!"<<std::endl;
	return 0;
}