#include <xmdv/CHierDimstack.h>
#include <xmdv/ogl.h>
#include <xmdv/cluster.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/colormap.h>
#include <xmdv/magnify.h>
#include <xmdv/brush_oper.h>
#include <xmdv/CFlatDimstack.h>

CHierDimstack::CHierDimstack(CHierDisplayInformation* hier_info) : CHierDisplay(hier_info)
{
}


//draw the mean and band of one cluster
void CHierDimstack::DrawSingleCluster(const Cluster &c, bool brushed)
{
	for ( int i = 0; i < g_globals.dataset->dims; i++ )
	{
		mean[i] = c.mean[i];
	}

	//draw band
    if ( hier_info->SHOW_BANDS )
	{
		DrawSingleNodeBand(c, hier_info->density, brushed);
	}
    
	//draw mean
    if ( hier_info->SHOW_MEANS )
    {
        if ( brushed )
	    {
            canvas->SetForeground(interface_color[SBB], current_opacity);
            glLineWidth(1.5);
        } 
	    else 
	    {
            int color_idx = int(c.color*(CMAPSIZE-1));
            canvas->SetForeground( (*hier_info->CMAP)[color_idx], current_opacity);
            glLineWidth(1.0);
        }

	    //draw the mean
        DrawSingleDimstackBin(canvas,mean,g_globals.screen_distortion->dimstack_size);
    }
}


//
// Compute the descretized value from an original data value for a particular
// dimension.  Display will be messed up if it is out of bounds.  You then
// should fix the range for that dimension in your data file.
//
static int get_contrib(int ind, double data)
{
    int res = int( ((data - g_globals.dataset->dim_min[ind]) / (g_globals.dataset->dim_max[ind] - g_globals.dataset->dim_min[ind])) *
              (double)(g_globals.dataset->cardinality[ind]));

    if (res >= g_globals.dataset->cardinality[ind] )
	{
		res--;
	}
	else if ( res < 0) 
	{
		res++;
	}
/*
    if (res >= g_globals.dataset->cardinality[ind] || res < 0) {
        printf("Data point out of bounds: dimension %d: [%f, %f]: %f\n", ind,
            g_globals.dataset->dim_min[ind], g_globals.dataset->dim_max[ind], data);
    }
*/
    return res;
}


void CHierDimstack::DrawSingleDimstackBin(grCanvas *canvas, const ARR<double> &data, const ARR<double> &size)
{
    int j;
    for (j=0; j<g_globals.dataset->dims; j++)
        contrib[j] = get_contrib(j, data[j]);

    double endx = 0, endy = 0;
    for (j=0; j<g_globals.dataset->dims; j++)
    {
        endx += contrib[j] * size[j];
        j++;
        if (j < g_globals.dataset->dims)
            endy += contrib[j] * size[j];
    }
    canvas->FillRectangle(endx, g_globals.screen_distortion->dimstack_Y_size - endy - g_globals.screen_distortion->dimstack_Y_blocksize, 
						  g_globals.screen_distortion->dimstack_X_blocksize, g_globals.screen_distortion->dimstack_Y_blocksize);
}


//calculate the distance factor of a bin in the band
double CHierDimstack::distanceFactor()
{
	double temp, temp1;
	double factor, sum = 0;
	for ( int i = 0; i < g_globals.dataset->dims; i++ )
	{
		if ( contrib[i] > contrib0[i] )
		{
			temp = contrib[i] - contrib0[i];
			temp1 = contrib2[i] - contrib0[i];
		}
		else
		{
			temp = contrib0[i] - contrib[i];
			temp1 = contrib0[i] - contrib1[i];
		}
		if ( temp1 > 0 )
			factor = temp / temp1;
		else
			factor = 0;
		sum += factor;
	}
	factor = sum / g_globals.dataset->dims;
	if ( factor < 0.0 )
		factor = -1.0 * factor;

	if ( fabs( factor ) < 1e-6 )
		return 0; 
	else
		return ((1-factor)*0.9+0.1)*alpha;
}


//draw a bin in a band
void CHierDimstack::DrawSingleDimstackBin2()
{
	double density = distanceFactor();
	canvas->SetForeground( color, density );

    double endx = 0, endy = 0;
    for (int j=0; j<g_globals.dataset->dims; j++)
    {
        endx += contrib[j] * g_globals.screen_distortion->dimstack_size[j];
        j++;
        if (j < g_globals.dataset->dims)
            endy += contrib[j] * g_globals.screen_distortion->dimstack_size[j];
    }
    canvas->FillRectangle(endx, g_globals.screen_distortion->dimstack_Y_size - endy - g_globals.screen_distortion->dimstack_Y_blocksize, 
						  g_globals.screen_distortion->dimstack_X_blocksize, g_globals.screen_distortion->dimstack_Y_blocksize);
}


//draw band of level k
void CHierDimstack::traceBand(int k)
{
	int j;
	if ( k == g_globals.dataset->dims-1 )
	{
		for ( j = contrib1[k]; j <= contrib2[k]; j++ )
		{
			contrib[k] = j;
			DrawSingleDimstackBin2();
		}
		return;
	}
	else
	{
		for ( j = contrib1[k]; j <= contrib2[k]; j++ )
		{
			contrib[k] = j;
			traceBand(k+1);
		}
		return;
	}
}

//draw a complete band
void CHierDimstack::hierDimstackDrawSingleNodeBand2(const ARR<double> &data0, const ARR<double> &data1, const ARR<double> &data2)
{
    int j;
    for (j=0; j<g_globals.dataset->dims; j++)
        contrib0[j] = get_contrib(j, data0[j]);
    for (j=0; j<g_globals.dataset->dims; j++)
        contrib1[j] = get_contrib(j, data1[j]);
    for (j=0; j<g_globals.dataset->dims; j++)
        contrib2[j] = get_contrib(j, data2[j]);

	traceBand(0);
}


//draw a band for one cluster
void CHierDimstack::DrawSingleNodeBand(const Cluster &c, double density, bool brushed)
{
    //if the cluster has only one children, there is no band.
    if ( c.num_children == 0 ) 
        return;

    //Set color
    if (hier_info->COLORED_BANDS) 
    {

        int color_idx = (int)(c.color*(CMAPSIZE-1));
        canvas->SetForeground( (*hier_info->CMAP)[color_idx], current_opacity);
    }
    else
        canvas->SetForeground(interface_color[BRUSH1+2]);

	for ( int i = 0; i < g_globals.dataset->dims; i++ )
	{
		max[i] = (c.max[i]-c.mean[i])*hier_info->extent_scaling_fac + c.mean[i];
		min[i] = (c.min[i]-c.mean[i])*hier_info->extent_scaling_fac + c.mean[i];
	}
	
	color = canvas->fg_color;
	alpha = canvas->alpha;
	hierDimstackDrawSingleNodeBand2(mean, min, max);
}


void CHierDimstack::SetParameters()
{
    CHierDisplay::SetParameters();
    mean.SetLen(g_globals.dataset->dims);
    max.SetLen(g_globals.dataset->dims);
    min.SetLen(g_globals.dataset->dims);

    contrib0.SetLen(g_globals.dataset->dims);
    contrib1.SetLen(g_globals.dataset->dims);
    contrib2.SetLen(g_globals.dataset->dims);
    contrib.SetLen(g_globals.dataset->dims);
}


void CHierDimstack::FreeSpace()
{
}

void CHierDimstack::DrawAxes()
{
	CFlatDimstack::DrawDimstackAxes(canvas);
}


void CHierDimstack::InputMouseMotion(int x, int y)
{
    // Hack to disable multiselect in HPC
    if (canvas->shiftkey && canvas->button[0] == true) return;

	CFlatDimstack::InputDimstackMouseMotion(canvas,x,y);
}

