#include <xmdv/CHierGlyph.h>
#include <xmdv/CFlatGlyph.h>
#include <xmdv/ogl.h>
#include <xmdv/cluster.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/colormap.h>
#include <xmdv/magnify.h>
#include <xmdv/brush_oper.h>

CHierGlyph::CHierGlyph(CHierDisplayInformation* hier_info) : CHierDisplay(hier_info)
{
	icos = NULL;
	isin = NULL;
}

CHierGlyph::~CHierGlyph()
{
	FreeSpace();
}

//draw the mean and band of one cluster
void CHierGlyph::DrawSingleCluster(const Cluster &c, bool brushed)
{
    basex = blocksize * (count%blockcnt) + halfblocksize;
    basey = blocksize * (count/blockcnt) + halfblocksize;
	count ++;

    int i;
    //draw the mean of the cluster
	double temp;
	for ( i = 0; i < dims; i++ )
	{
        temp = (c.mean[i] - g_globals.dataset->dim_min[i]);
		polymean[i].X = basex + temp * icos[i];
        polymean[i].Y = basey - temp * isin[i];;
	}

    
    //draw band
    if ( hier_info->SHOW_BANDS )
		DrawSingleNodeBand(c, hier_info->density, brushed);

    //draw mean
    if ( hier_info->SHOW_MEANS )
    {
        if ( brushed )
            canvas->SetForeground(interface_color[SBB], current_opacity);
	    if ( ! brushed )
	    {
            int color_idx = int(c.color*(CMAPSIZE-1));
            canvas->SetForeground( (*hier_info->CMAP)[color_idx], current_opacity);
        }

        for (i =0; i < dims; i++) 
		{
			canvas->DrawLine((float) basex, (float) basey,
							 (float) polymean[i].X, (float) polymean[i].Y);
		}

        for (i =0; i < dims-1; i++) 
	    {
		    canvas->DrawLine((float) polymean[i].X, (float) polymean[i].Y,
							 (float) polymean[i+1].X, (float) polymean[i+1].Y);
        }
		canvas->DrawLine((float) polymean[0].X, (float) polymean[0].Y,
					     (float) polymean[dims-1].X, (float) polymean[dims-1].Y);
    }
    
}

//draw a band for one cluster
void CHierGlyph::DrawSingleNodeBand(const Cluster &c, double density, bool is_brushed)
{
    //if the cluster has only one children, there is no band.
    if ( c.num_children == 0 ) 
        return;
	
    //Set color
    if (hier_info->COLORED_BANDS) 
    {
        int color_idx = (int)(c.color*(CMAPSIZE-1));
        canvas->SetForeground( (*hier_info->CMAP)[color_idx], current_opacity);
    }
    else
        canvas->SetForeground(interface_color[BRUSH1+2]);

	double temp1, temp2, temp3;
	int i;

	polymean[dims] = polymean[0];
	
	for ( i = 0; i < dims; i++ )
	{
        temp1 = (c.mean[i] - g_globals.dataset->dim_min[i]);
        temp2 = (c.max[i] - g_globals.dataset->dim_min[i]);
        temp3 = (c.min[i] - g_globals.dataset->dim_min[i]);
		if ( fabs(1 - hier_info->extent_scaling_fac ) > 1e-4 )
		{
			temp2 = temp1 + hier_info->extent_scaling_fac*(temp2-temp1);
			temp3 = temp1 + hier_info->extent_scaling_fac*(temp3-temp1);
        }
		polymax[i].X = basex + temp2 * icos[i];
        polymax[i].Y = basey - temp2 * isin[i];;
		polymin[i].X = basex + temp3 * icos[i];
        polymin[i].Y = basey - temp3 * isin[i];;
	}
	polymax[dims] = polymax[0];
	polymin[dims] = polymin[0];


    // Draw the lower band
    int d;
    for (d=0; d<dims+1; d++)
    {
		Vec2 lo, hi;

        lo = polymin[d];

		hi = polymean[d];

//        lo.Y = (1-extent_scaling_fac)*hi.Y + (extent_scaling_fac*lo.Y);

        if (d==0) {
            band[0] = lo;
            band[3] = hi;

        } else {
	
            band[1] = lo;
            band[2] = hi;
            canvas->DrawTransparentBand(band, 0.0, density, canvas->fg_color, canvas->fg_color);

            band[0] = band[1];
            band[3] = band[2];
        }
    }

    // Draw the upper band
    for (d=0; d<dims+1; d++)
    {
        Vec2 lo, hi;

		lo = polymean[d];
		hi = polymax[d];

        // Calculate how much to scale, if any
//        hi.Y = (1-extent_scaling_fac)*lo.Y + (extent_scaling_fac*hi.Y);

        if (d==0) {
            band[0] = lo;
            band[3] = hi;
        } else {
            band[1] = lo;
            band[2] = hi;

            canvas->DrawTransparentBand(band, density, 0.0, canvas->fg_color, canvas->fg_color);

            band[0] = band[1];
            band[3] = band[2];
        }
    }
}



void CHierGlyph::CountBrushedClusters( const Cluster &node )
{
    if ( ! BITTEST(node.flags,IS_BRUSHED_ANY) )
        return;

    if ( BITTEST(node.flags,IS_CONTOUR1) )
    {
        if ( BITTEST(node.flags,IS_BRUSHED) )
            brushedClusterNumber++;
        else 
            return;
    }
    else
    {
        for (int j=0; j<node.num_children; j++) 
            CountBrushedClusters(g_globals.btree->nodes[node.child[j]]);
    }
}


void CHierGlyph::CountUnbrushedClusters( const Cluster &node )
{
	if ( BrushSemantic == 0 )
	{
		 if ( BITTEST(node.flags,IS_BRUSHED) )
			return;
	}

    if ( BITTEST(node.flags,IS_CONTOUR0) )
	{
        if ( ! BITTEST(node.flags,IS_BRUSHED) )
            unbrushedClusterNumber++;
        else 
            return;
	}
    else
    {
        for (int j=0; j<node.num_children; j++) 
            CountUnbrushedClusters(g_globals.btree->nodes[node.child[j]]);
    }
}



void CHierGlyph::SetParameters()
{
    CHierDisplay::SetParameters();
	band.SetLen(4);
	int nodeNumber = 0, i; 

    brushedClusterNumber = 0;
    unbrushedClusterNumber = 0;
	if (!MAGNIFY)
	{
        CountUnbrushedClusters(g_globals.btree->nodes[0]);
        CountBrushedClusters(g_globals.btree->nodes[0]);
        nodeNumber = brushedClusterNumber + unbrushedClusterNumber; 
	}
    else
    {
        CountBrushedClusters(g_globals.btree->nodes[0]);
        nodeNumber = brushedClusterNumber;
    }
    
	dims = g_globals.dataset->dims;
	//set canvas arrangement parameters
	blocksize = 1.0/(1.0 + sqrt((double)nodeNumber));
    blockcnt = int( .5 + sqrt((double)nodeNumber) );
    halfblocksize = .5 * blocksize;
    count = 0;

    alpha = 2.0*M_PI/dims;
	isin = new double[dims];
	icos = new double[dims];
    
	double temp;
	for (i=0; i<dims; i++)
    {
        temp = (blocksize/2.0) / (g_globals.dataset->dim_max[i] - g_globals.dataset->dim_min[i]);
		icos[i] = temp * cos(i*alpha);
        isin[i] = temp * sin(i*alpha);
	}

	polymean.SetLen(dims+1);
	polymax.SetLen(dims+1);
	polymin.SetLen(dims+1);

    glLineWidth((float) DSIZE);
}


void CHierGlyph::FreeSpace()
{
	if (icos) { delete [] icos; icos = NULL; }
	if (isin) { delete [] isin; isin = NULL; }
}


void CHierGlyph::InputMouseMotion(int x, int y)
{
    // Hack to disable multiselect in HPC
    if (canvas->shiftkey && canvas->button[0] == true) return;

//    CFlatGlyph::InputGlyphMouseMotion(canvas,x,y);
}
