#include <xmdv/CScreenDistortion.h>
#include <xmdv/XmdvTool.h>
#include <math.h>
#include <xmdv/CDimReducer.h>

CScreenDistortion::CScreenDistortion()
{
    distortion_status = false;
    spacing = true;
}


CScreenDistortion::~CScreenDistortion()
{

}



void CScreenDistortion::Reset()
{
    if ( !spacing )
    {
        ParacoordReset();
        ScatterReset();
        GlyphReset();
	    DimstackReset();
  //    InterRingReset();
    }
    else
    {
        if ( (g_globals.dim_reducer != NULL) && (g_globals.dim_reducer->status == APPLY) )
        {
            double *spacing_distances;
            spacing_distances = new double[g_globals.dataset->dims];
            g_globals.dim_reducer->GetSpacingDistances(spacing_distances);
            ParacoordResetWithSpacing(spacing_distances);
            ScatterReset();
            GlyphResetWithSpacing(spacing_distances);
	        DimstackReset();

            delete [] spacing_distances;
        }
        else
        {
            ParacoordReset();
            ScatterReset();
            GlyphReset();
	        DimstackReset();
        }


    }
}

void CScreenDistortion::ParacoordResetWithSpacing(double *distances)
{
    if (!g_globals.dataset) 
        return;

	if ( g_globals.dataset->dims < 2 )
		return;
    
    int i;

    paracoord_axes.SetLen(g_globals.dataset->dims);
    paracoord_interval.SetLen(g_globals.dataset->dims);
    

    double ave_width = 1./g_globals.dataset->dims;
    double total = 0;
    for ( i = 0; i < g_globals.dataset->dims-1; i++ )
    {
        paracoord_interval[i] = distances[i];
        total += distances[i];
    }

    for ( i = 0; i < g_globals.dataset->dims-1; i++ )
    {
        paracoord_interval[i] = paracoord_interval[i] / total * (1.-ave_width);
    }

    paracoord_axes[0] = ave_width/2.;
    for ( i = 0; i < g_globals.dataset->dims-1; i++ )
    {
        paracoord_axes[i+1] = paracoord_axes[i] +  paracoord_interval[i];
    }
  	paracoord_interval[i] = 1-paracoord_axes[i];
}


void CScreenDistortion::ParacoordReset()
{
    if (!g_globals.dataset) 
        return;

	if ( g_globals.dataset->dims < 2 )
		return;
//reset screen distortion parameters for paracoord display
    paracoord_axes.SetLen(g_globals.dataset->dims);
    paracoord_interval.SetLen(g_globals.dataset->dims);
    
    double width = 1./g_globals.dataset->dims;
    paracoord_axes[0] = width/2.;

    int i;

    for ( i = 0; i < g_globals.dataset->dims-1; i++ )
    {
        paracoord_axes[i+1] = paracoord_axes[i] +  width;
        paracoord_interval[i] = width;
    }
	paracoord_interval[i] = 1-paracoord_axes[i];
}


void CScreenDistortion::ParacoordMapIntervalToAxes(ARR<double> &axes, ARR<double> &interval)
{
    axes[0] = 0.5/g_globals.dataset->dims; 
    for ( int i = 1; i < g_globals.dataset->dims-1; i++ )
    {
        axes[i] = axes[i-1] + interval[i-1];
    }
    axes[g_globals.dataset->dims - 1] = 1. - 0.5/g_globals.dataset->dims;
}

  
void CScreenDistortion::ParacoordScaleAxes(int dim, ARR<double> &axes, ARR<double> &interval, double delta)
{
    double min_width = 1./g_globals.dataset->dims * 0.3;
    double max_width = 0.9;//?
    if ( (interval[dim] + delta) > max_width )
        delta = max_width - interval[dim];
    else if ( (interval[dim] + delta) < min_width )
        delta = min_width - interval[dim];

    //left is the width of the screen minus the blank space.
    double left = 1. - 1./g_globals.dataset->dims;
    double alpha = left - delta/(left - interval[dim]);
    double total = 0.;
    for ( int i = 0; i < g_globals.dataset->dims-1; i++ )
    {
        if ( i != dim )
        {
            interval[i] *= alpha;
            if ( interval[i] < min_width )
                interval[i] = min_width;
            total += interval[i];
        }
    }

    interval[dim] = left - total;

    ParacoordMapIntervalToAxes(axes, interval);
}

void CScreenDistortion::ParacoordScaleAxes(int dim, double delta)
{
    if ( g_globals.dataset->dims <= 2 )
        return;

    ParacoordScaleAxes(dim, paracoord_axes, paracoord_interval, delta);
    return;
}




/*************************************Scatterplot Matrixes************************************************/
/* for fisheye view distortion
void CScreenDistortion::ScatterReset()
{
    if (!g_globals.dataset) 
        return;

//reset screen distortion parameters for dimemsion stacking display
    scatter_X_axes.SetLen(g_globals.dataset->dims+1);
    scatter_Y_axes.SetLen(g_globals.dataset->dims+1);
    scatter_X_interval.SetLen(g_globals.dataset->dims);
    scatter_Y_interval.SetLen(g_globals.dataset->dims);
    
    double width = 1./g_globals.dataset->dims;
    scatter_X_axes[0] = scatter_Y_axes[0] = 0;
    int i;
	for ( i = 0; i < g_globals.dataset->dims; i++ )
    {
        scatter_X_axes[i+1] = scatter_Y_axes[i+1] = scatter_X_axes[i] +  width;
        scatter_X_interval[i] = scatter_Y_interval[i] = width;
    }
	
	double x;
	double d = 2.5;
	for ( i = 0; i <= g_globals.dataset->dims; i++ )
    {
		x = scatter_X_axes[i]*2-1;
		if ( x > 0 )
		{
			x = (d+1)*x/(d*x+1);
			x = (x+1)/2;
			scatter_X_axes[i] = scatter_Y_axes[i] = x;
		}
		else
		{
			x *= -1;
			x = (d+1)*x/(d*x+1);
			x = (-1*x+1)/2;
			scatter_X_axes[i] = scatter_Y_axes[i] = x;
		}
    }
	
	for ( i = 0; i < g_globals.dataset->dims; i++ )
	{
        scatter_X_interval[i] = scatter_Y_interval[i] = scatter_X_axes[i+1] - scatter_X_axes[i];

	}

}
*/


void CScreenDistortion::ScatterReset()
{
    if (!g_globals.dataset) 
        return;

	if ( g_globals.dataset->dims < 1 )
		return;

//reset screen distortion parameters for dimemsion stacking display
    scatter_X_axes.SetLen(g_globals.dataset->dims+1);
    scatter_Y_axes.SetLen(g_globals.dataset->dims+1);
    scatter_X_interval.SetLen(g_globals.dataset->dims);
    scatter_Y_interval.SetLen(g_globals.dataset->dims);
    
    double width = 1./g_globals.dataset->dims;
    scatter_X_axes[0] = scatter_Y_axes[0] = 0;
    for ( int i = 0; i < g_globals.dataset->dims; i++ )
    {
        scatter_X_axes[i+1] = scatter_Y_axes[i+1] = scatter_X_axes[i] +  width;
        scatter_X_interval[i] = scatter_Y_interval[i] = width;
    }
}


void CScreenDistortion::ScatterMapIntervalToAxes(ARR<double> &axes, ARR<double> &interval)
{
    axes[0] = 0.; 
    for ( int i = 1; i < g_globals.dataset->dims; i++ )
    {
        axes[i] = axes[i-1] + interval[i-1];
    }
    axes[g_globals.dataset->dims] = 1.;
}

void CScreenDistortion::ScatterScalePlotInOneDimension(int dim, ARR<double> &axes, ARR<double> &interval, double delta)
{
    double min_width = 1./g_globals.dataset->dims * 0.3;
    if ( (interval[dim] + delta) > 0.8 )
        delta = 0.8 - interval[dim];
    else if ( (interval[dim] + delta) < min_width )
        delta = min_width - interval[dim];

    double alpha = 1 - delta/(1. - interval[dim]);
    double total = 0.;
    for ( int i = 0; i < g_globals.dataset->dims; i++ )
    {
        if ( i != dim )
        {
            interval[i] *= alpha;
            if ( interval[i] < min_width )
                interval[i] = min_width;
            total += interval[i];
        }
    }

    interval[dim] = 1. - total;

    ScatterMapIntervalToAxes(axes, interval);

    /*    double temp = interval[dim] * alpha;
    if ( temp > 0.8 )
        temp = 0.8;
    
    double other_interval = (1.- temp)/(g_globals.dataset->dims-1);
    for ( int i = 0; i < g_globals.dataset->dims; i++ )
        interval[i] = other_interval;

    interval[dim] = temp;

    MapIntervalToAxes(axes, interval);
*/
    /*    int i;
    int begin_dim, end_dim;
    double other_interval; 
    
    if ( dim == 0 || dim == g_globals.dataset->dims-1 )
    {
        if ( dim == 0 )
        {
            begin_dim = 1;
            end_dim = g_globals.dataset->dims-1;
        }
        else 
        {
            begin_dim = 0;
            end_dim = g_globals.dataset->dims-2;
        }
    
        interval[dim] *= alpha;
        if ( interval[dim] > 0.8 )
            interval[dim] = 0.8;
        other_interval = (1.- interval[dim])/(g_globals.dataset->dims-1);
        for ( i = begin_dim; i <= end_dim; i++ )
            interval[i] = other_interval;
    }
    else
    {
        double mid;
        mid = axes[dim] + interval[dim]/2.;

        interval[dim] *= alpha;
        if ( interval[dim] > 0.8 )
            interval[dim] = 0.8;
        
        begin_dim = 0;
        end_dim = dim-1;
        other_interval = (mid - interval[dim]/2)/(end_dim+1);
        for ( i = begin_dim; i <= end_dim; i++ )
            interval[i] = other_interval;
        
        begin_dim = dim+1;
        end_dim = g_globals.dataset->dims-1;
        other_interval = (1 - mid - interval[dim]/2)/(g_globals.dataset->dims - end_dim + 1);
        for ( i = begin_dim; i <= end_dim; i++ )
            interval[i] = other_interval;
    }
    
    MapIntervalToAxes(axes, interval);
    */
}

void CScreenDistortion::ScatterScalePlot(int X_dim, int Y_dim, double delta)
{
    if ( g_globals.dataset->dims <= 2 )
        return;

    ScatterScalePlotInOneDimension(X_dim, scatter_X_axes, scatter_X_interval, delta);
    ScatterScalePlotInOneDimension(Y_dim, scatter_Y_axes, scatter_Y_interval, delta);
    return;
}


/*************************************dimensional stacking************************************************/

//
// At each level, the number of pixels between successive values for the
// dimension involved varies.  It depends on the minimum block size and
// the cardinalities of the dimensions embedded within it.  This in turn
// depends on the orientation and speed of each dimension.
//
void CScreenDistortion::DimstackReset()
{
    if (!g_globals.dataset) 
        return;

	if ( g_globals.dataset->dims < 2 )
		return;

    dimstack_size.SetLen(g_globals.dataset->dims);

    double odd=1, even=1;

    // Compute product of cardinalities in each orientation
    int i;
    for (i = 0;i < g_globals.dataset->dims;i=i+2)
        odd *= g_globals.dataset->cardinality[i];

    for (i = 1;i < g_globals.dataset->dims;i=i+2)
        even *= g_globals.dataset->cardinality[i];

    odd = 1.0/odd;
    even = 1.0/even;

    // Now compute size of a virtual pixel for each dimension
    for (i = 0;i < g_globals.dataset->dims;i++)   {
        dimstack_size[i] = (i&1)?even:odd;
        for (int j = i+2; j < g_globals.dataset->dims; j=j+2)
            dimstack_size[i] *= g_globals.dataset->cardinality[j];
    }

    dimstack_X_blocksize = odd;
    dimstack_Y_blocksize = even;
    dimstack_X_size = dimstack_size[0]*g_globals.dataset->cardinality[0];
    dimstack_Y_size = dimstack_size[1]*g_globals.dataset->cardinality[1];
}


void CScreenDistortion::InterRingReset()
{
    if ( g_globals.dtree != NULL )
        g_globals.dtree->ResetAngles();
}   


void CScreenDistortion::GlyphReset()
{
    if (!g_globals.dataset) 
        return;

	if ( g_globals.dataset->dims < 1 )
		return;

    link = true;
    threshold = 0.2;
    sort_dim = 0;
    link_method = 1;



    int j;
    double alpha = 2.0*M_PI/g_globals.dataset->dims;

    star_angles.SetLen(g_globals.dataset->dims);
    star_angles_cos_sized.SetLen(g_globals.dataset->dims);
    star_angles_sin_sized.SetLen(g_globals.dataset->dims);

    for (j=0; j<g_globals.dataset->dims; j++) 
    {
        star_angles[j] = j*alpha;
        star_angles_cos_sized[j] = (g_globals.dataset->blocksize/2.0) * cos(star_angles[j]);
        star_angles_sin_sized[j] = (g_globals.dataset->blocksize/2.0) * sin(star_angles[j]);
    }

    double del_alpha = 0.1 * alpha;

/*    min_alpha.SetLen(g_globals.dataset->dims);
    min_sized_cos_alpha.SetLen(g_globals.dataset->dims);
    min_sized_sin_alpha.SetLen(g_globals.dataset->dims);
    
    max_alpha.SetLen(g_globals.dataset->dims);
    max_sized_cos_alpha.SetLen(g_globals.dataset->dims);
    max_sized_sin_alpha.SetLen(g_globals.dataset->dims);

    for (j=0; j<g_globals.dataset->dims; j++) 
    {
        min_alpha[j] = star_angles[j] - del_alpha;
        min_sized_cos_alpha[j] = (g_globals.dataset->blocksize/2.0) * cos(min_alpha[j]);
        min_sized_sin_alpha[j] = (g_globals.dataset->blocksize/2.0) * sin(min_alpha[j]);

        max_alpha[j] = star_angles[j] + del_alpha;
        max_sized_cos_alpha[j] = (g_globals.dataset->blocksize/2.0) * cos(max_alpha[j]);
        max_sized_sin_alpha[j] = (g_globals.dataset->blocksize/2.0) * sin(max_alpha[j]);
    }  */
}


void CScreenDistortion::GlyphResetWithSpacing(double *distances)
{
    if (!g_globals.dataset) 
        return;

	if ( g_globals.dataset->dims < 1 )
		return;

    link = true;
    threshold = 0.2;
    sort_dim = 0;
    link_method = 1;

    int i, j;
    double alpha = 2.0*M_PI/g_globals.dataset->dims;

    star_angles.SetLen(g_globals.dataset->dims);
    star_angles_cos_sized.SetLen(g_globals.dataset->dims);
    star_angles_sin_sized.SetLen(g_globals.dataset->dims);

    double total = 0;
    for ( i = 0; i < g_globals.dataset->dims; i++ )
    {
        total += distances[i];
    }

    star_angles[0] = 0;
    for ( i = 1; i < g_globals.dataset->dims; i++ )
    {
        star_angles[i] = star_angles[i-1] + distances[i] * 2.0 * M_PI / total; 
    }
    
    for (j=0; j<g_globals.dataset->dims; j++) 
    {
        star_angles_cos_sized[j] = (g_globals.dataset->blocksize/2.0) * cos(star_angles[j]);
        star_angles_sin_sized[j] = (g_globals.dataset->blocksize/2.0) * sin(star_angles[j]);
    }


}
