
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <stdio.h>

#include <xmdv/cluster.h>
#include <xmdv/XmdvTool.h>
#include <xmdv/brush.h>
#include <xmdv/CHierNav.h>
#include <xmdv/CHierHist.h>
#include <xmdv/CHierTreemap.h>
#include <xmdv/OnlineDataClustering.h>

/* SID */
#include <backend/db_buffer.h>

bool ClusterTree::ReadClusterTreeFile(const char* fname)
{
	
    int k, i;
    assert(g_globals.dataset);
   
    //test if file exists. If not, online generate one.
    FILE *fptr = fopen(fname,"rb");
    if (fptr==NULL) {
        OnlineClusterOkcFileToCgFile(g_globals.dataset->datafile, 100, true);
        fptr = fopen(fname,"rb");
        if (fptr==NULL) 
            return false;
    }

    //test if binary file.
    unsigned int magic_code;
    fread(&magic_code, sizeof(unsigned int), 1, fptr);
    
    fclose(fptr);

    if ( magic_code == 0xcecfcecf )
    {
        return ReadBinaryClusterTreeFile(fname);
    }
    
    fptr = fopen(fname,"r");

    clusterfile = strdup(fname);

    // Get file size
    int num_nodes;
    if (fscanf(fptr, "%d %d\n", &num_nodes, &dimension)<=0)
    {
        printf("Error reading the first line of the file %s\n", fname);
        printf("Format of the first line: #nodes #dimension\n");
        exit(-1);
    }
    assert(dimension == g_globals.dataset->all_dims);

    // Allocate space for all nodes
    //
    nodes.SetLen(num_nodes);

//  selected.SetLen(num_nodes/2+1);

//    printf("Number of cluster nodes=%d\n", num_nodes);


    for (i=0; i<num_nodes; i++) {
        nodes[i].sx.SetLen(dimension);
        nodes[i].mean.SetLen(dimension);
        nodes[i].max.SetLen(dimension);
        nodes[i].min.SetLen(dimension);
      //  nodes[i].all_mean.SetLen(dimension);
        nodes[i].num_children = 0;
        nodes[i].entries = 0;
    }

    for (k=0; k<num_nodes; k++ )
        for (i=0;i<dimension; i++)
            nodes[k].sx[i] = 0.0;

// SID 

    double max, min;
    for (k=0; k<num_nodes; k++)
    {
        int id;
        fscanf(fptr, "%d %d %d", &id, &nodes[k].parent, &nodes[k].entries);

        for (i=0; i<dimension; i++)
		{
			//Read the actual value of the dimension cluster and store in the node
            fscanf(fptr, "%lf ", &(nodes[k].sx[i]));

            // Compute the mean
            if ( nodes[k].entries > 0) {

                double sum_dim = nodes[k].sx[i] * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) +
                                 (nodes[k].entries * g_globals.dataset->cg_dim_min[i]);

                nodes[k].mean[i] = sum_dim/nodes[k].entries;
            }

        }


        fscanf(fptr, "%lf ", &nodes[k].sqrt_radius);

        for (i=0; i<dimension; i++)
		{
            fscanf(fptr, "%lf ", &max);
			nodes[k].max[i] = max * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) + g_globals.dataset->cg_dim_min[i];
		}

        for (i=0; i<dimension; i++)
		{
            fscanf(fptr, "%lf ", &min);
			nodes[k].min[i] = min * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) + g_globals.dataset->cg_dim_min[i];
		}

        fscanf(fptr, "\n");

        if ( nodes[k].parent >= 0 ) {
            nodes[nodes[k].parent].child.InsertTail(k);
            nodes[nodes[k].parent].num_children++;
        }

    }

    fclose(fptr);

    // Init the min/max extents
    //
    min_extents.SetLen(g_globals.dataset->all_dims);
    max_extents.SetLen(g_globals.dataset->all_dims);

    for (int d=0; d<g_globals.dataset->all_dims; d++) {
        min_extents[d] = HUGE_VAL;
        max_extents[d] = -HUGE_VAL;
    }

    treemap_limit_number = (int) (nodes[0].entries * TREEMAP_LIMIT);
    if ( treemap_limit_number < 1 )
        treemap_limit_number = 1;
    return true;
}



bool ClusterTree::ReadBinaryClusterTreeFile(const char* fname)
{
    int k, i;
    assert(g_globals.dataset);
   
    FILE *fptr = fopen(fname,"rb");
    if (fptr==NULL) 
        return false;

    unsigned int magic_code;
    fread(&magic_code, sizeof(unsigned int), 1, fptr);

    clusterfile = strdup(fname);

    // Get file size
    int num_nodes;
    if ( fread(&num_nodes, sizeof(int), 1, fptr) != 1 || fread(&dimension, sizeof(int), 1, fptr) != 1 ) 
    {
        printf("Format of the first line: #nodes #dimension\n");
        exit(-1);
    }
    assert(dimension == g_globals.dataset->all_dims);

    // Allocate space for all nodes
    //
    nodes.SetLen(num_nodes);

//  selected.SetLen(num_nodes/2+1);

    printf("Number of cluster nodes=%d\n", num_nodes);


    for (i=0; i<num_nodes; i++) {
        nodes[i].sx.SetLen(dimension);
        nodes[i].mean.SetLen(dimension);
        nodes[i].max.SetLen(dimension);
        nodes[i].min.SetLen(dimension);
      //  nodes[i].all_mean.SetLen(dimension);
        nodes[i].num_children = 0;
        nodes[i].entries = 0;
    }

    for (k=0; k<num_nodes; k++ )
        for (i=0;i<dimension; i++)
            nodes[k].sx[i] = 0.0;

// SID 

    double * buffer;
    buffer = new double[dimension];
    for (k=0; k<num_nodes; k++)
    {
        int id;
        fread(&id, sizeof(int), 1, fptr);
        fread(&nodes[k].parent, sizeof(int), 1, fptr);
        fread(&nodes[k].entries, sizeof(int), 1, fptr);


        fread(buffer, sizeof(double), dimension, fptr);
        for (i=0; i<dimension; i++)
		{
              nodes[k].sx[i] = buffer[i];
//            fread(&(nodes[k].sx[i]), sizeof(double), 1, fptr);
//            fscanf(fptr, "%lf ", &(nodes[k].sx[i]));

            // Compute the mean
            if ( nodes[k].entries > 0) {

//                double sum_dim = nodes[k].sx[i] * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) +
//                                 (nodes[k].entries * g_globals.dataset->cg_dim_min[i]);
                double sum_dim = buffer[i] * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) +
                                 (nodes[k].entries * g_globals.dataset->cg_dim_min[i]);

                nodes[k].mean[i] = sum_dim/nodes[k].entries;
            }

        }


        fread(&(nodes[k].sqrt_radius), sizeof(double), 1, fptr);
//        fscanf(fptr, "%lf ", &nodes[k].sqrt_radius);

        fread(buffer, sizeof(double), dimension, fptr);
        for (i=0; i<dimension; i++)
		{
//            fread(&max, sizeof(double), 1, fptr);
//            fscanf(fptr, "%lf ", &max);
			nodes[k].max[i] = buffer[i] * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) + g_globals.dataset->cg_dim_min[i];
		}

        fread(buffer, sizeof(double), dimension, fptr);
        for (i=0; i<dimension; i++)
		{
//            fread(&min, sizeof(double), 1, fptr);
//            fscanf(fptr, "%lf ", &min);
			nodes[k].min[i] = buffer[i] * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) + g_globals.dataset->cg_dim_min[i];
		}

//        fscanf(fptr, "\n");

        if ( nodes[k].parent >= 0 ) {
            nodes[nodes[k].parent].child.InsertTail(k);
            nodes[nodes[k].parent].num_children++;
        }

    }

    fclose(fptr);
    delete [] buffer;

    // Init the min/max extents
    //
    min_extents.SetLen(g_globals.dataset->all_dims);
    max_extents.SetLen(g_globals.dataset->all_dims);

    for (int d=0; d<g_globals.dataset->all_dims; d++) {
        min_extents[d] = HUGE_VAL;
        max_extents[d] = -HUGE_VAL;
    }

    treemap_limit_number = (int) (nodes[0].entries * TREEMAP_LIMIT);
    if ( treemap_limit_number < 1 )
        treemap_limit_number = 1;

    return true;
}


/*
bool ClusterTree::ReadClusterTreeFile(const char* fname)
{
    int k, i;
    assert(g_globals.dataset);

    FILE *fptr = fopen(fname,"r");
    if (fptr==NULL) {
        OnlineClusterOkcFileToCgFile(g_globals.dataset->datafile, 100);
        fptr = fopen(fname,"r");
        if (fptr==NULL) 
            return false;
    }

    clusterfile = strdup(fname);

    // Get file size
    int num_nodes;
    if (fscanf(fptr, "%d %d\n", &num_nodes, &dimension)<=0)
    {
        printf("Error reading the first line of the file %s\n", fname);
        printf("Format of the first line: #nodes #dimension\n");
        exit(-1);
    }
    assert(dimension == g_globals.dataset->all_dims);

    // Allocate space for all nodes
    //
    nodes.SetLen(num_nodes);

//  selected.SetLen(num_nodes/2+1);

    printf("Number of cluster nodes=%d\n", num_nodes);


    for (i=0; i<num_nodes; i++) {
        nodes[i].sx.SetLen(dimension);
        nodes[i].mean.SetLen(dimension);
        nodes[i].max.SetLen(dimension);
        nodes[i].min.SetLen(dimension);
      //  nodes[i].all_mean.SetLen(dimension);
        nodes[i].num_children = 0;
        nodes[i].entries = 0;
    }

    for (k=0; k<num_nodes; k++ )
        for (i=0;i<dimension; i++)
            nodes[k].sx[i] = 0.0;

// SID 

    double max, min;
    for (k=0; k<num_nodes; k++)
    {
        int id;
        fscanf(fptr, "%d %d %d", &id, &nodes[k].parent, &nodes[k].entries);

        for (i=0; i<dimension; i++)
		{
            fscanf(fptr, "%lf ", &(nodes[k].sx[i]));

            // Compute the mean
            if ( nodes[k].entries > 0) {

                double sum_dim = nodes[k].sx[i] * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) +
                                 (nodes[k].entries * g_globals.dataset->cg_dim_min[i]);

                nodes[k].mean[i] = sum_dim/nodes[k].entries;
            }

        }


        fscanf(fptr, "%lf ", &nodes[k].sqrt_radius);

        for (i=0; i<dimension; i++)
		{
            fscanf(fptr, "%lf ", &max);
			nodes[k].max[i] = max * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) + g_globals.dataset->cg_dim_min[i];
		}

        for (i=0; i<dimension; i++)
		{
            fscanf(fptr, "%lf ", &min);
			nodes[k].min[i] = min * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) + g_globals.dataset->cg_dim_min[i];
		}

        fscanf(fptr, "\n");

        if ( nodes[k].parent >= 0 ) {
            nodes[nodes[k].parent].child.InsertTail(k);
            nodes[nodes[k].parent].num_children++;
        }

    }

    // Init the min/max extents
    //
    min_extents.SetLen(g_globals.dataset->all_dims);
    max_extents.SetLen(g_globals.dataset->all_dims);

    for (int d=0; d<g_globals.dataset->all_dims; d++) {
        min_extents[d] = HUGE_VAL;
        max_extents[d] = -HUGE_VAL;
    }

    treemap_limit_number = (int) (nodes[0].entries * TREEMAP_LIMIT);
    if ( treemap_limit_number < 1 )
        treemap_limit_number = 1;

    return true;
}
*/

bool ClusterTree::ReadClusterTreeFileWithDatabase(const char* fname)
{
    int k, i;
    assert(g_globals.dataset);

    FILE *fptr = fopen(fname,"r");
    if (fptr==NULL) {
        printf("Cluster file \"%s\" does not exist. Program terminated.\n", fname);
//        abort();
        return false;
    }

    clusterfile = strdup(fname);

    // Get file size
    int num_nodes;
    if (fscanf(fptr, "%d %d\n", &num_nodes, &dimension)<=0)
    {
        printf("Error reading the first line of the file %s\n", fname);
        printf("Format of the first line: #nodes #dimension\n");
        exit(-1);
    }
    assert(dimension == g_globals.dataset->all_dims);

    // Allocate space for all nodes
    //
    nodes.SetLen(num_nodes);

//  selected.SetLen(num_nodes/2+1);

    printf("Number of cluster nodes=%d\n", num_nodes);


    for (i=0; i<num_nodes; i++) {
        nodes[i].sx.SetLen(dimension);
        nodes[i].mean.SetLen(dimension);
      //  nodes[i].max.SetLen(dimension);
      //  nodes[i].min.SetLen(dimension);
        nodes[i].all_mean.SetLen(dimension);
        nodes[i].num_children = 0;
        nodes[i].entries = 0;
    }

/*    for (k=0; k<num_nodes; k++ )
        for (i=0;i<dimension; i++)
            nodes[k].sx[i] = 0.0;
*/
/* SID */

    double max, min, temp;
    for (k=0; k<num_nodes; k++)
    {
        int id;
        fscanf(fptr, "%d %d %d", &id, &nodes[k].parent, &nodes[k].entries);

        for (i=0; i<dimension; i++)
		{
            fscanf(fptr, "%lf ", &temp);

            // Compute the mean
            if ( nodes[k].entries > 0) {

                double sum_dim = nodes[k].sx[i] * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) +
                                 (nodes[k].entries * g_globals.dataset->cg_dim_min[i]);

                nodes[k].mean[i] = sum_dim/nodes[k].entries;
            }

        }


        fscanf(fptr, "%lf ", &nodes[k].sqrt_radius);

        for (i=0; i<dimension; i++)
		{
            fscanf(fptr, "%lf ", &max);
//			nodes[k].max[i] = max * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) + g_globals.dataset->cg_dim_min[i];
		}

        for (i=0; i<dimension; i++)
		{
            fscanf(fptr, "%lf ", &min);
//			nodes[k].min[i] = min * (g_globals.dataset->cg_dim_max[i]-g_globals.dataset->cg_dim_min[i]) + g_globals.dataset->cg_dim_min[i];
		}

        fscanf(fptr, "\n");

        if ( nodes[k].parent >= 0 ) {
            nodes[nodes[k].parent].child.InsertTail(k);
            nodes[nodes[k].parent].num_children++;
        }

    }

    // Init the min/max extents
    //
    min_extents.SetLen(g_globals.dataset->all_dims);
    max_extents.SetLen(g_globals.dataset->all_dims);

    for (int d=0; d<g_globals.dataset->all_dims; d++) {
        min_extents[d] = HUGE_VAL;
        max_extents[d] = -HUGE_VAL;
    }

    treemap_limit_number = (int) (nodes[0].entries * TREEMAP_LIMIT);
    if ( treemap_limit_number < 1 )
        treemap_limit_number = 1;

    return true;
}

//
// To mark the highlighted clusters based on the
// brush movement in other modes
//
void hierMarkBrushedClustersByExtents(Cluster* node)
{
    BITCLEAR(node->flags,IS_BRUSHED);
    BITCLEAR(node->flags,IS_BRUSHED_ANY);

    for (int i=0; i<node->num_children; i++)
        hierMarkBrushedClustersByExtents(&(g_globals.btree->nodes[node->child[i]]));

    // Reached the leaf node?
    //
    if (node->num_children == 0) 
	{
        int in=0;
        for (int k=0; k<g_globals.dataset->dims; k++) {
            if (node->mean[k] >= g_globals.btree->min_extents[k] && 
				node->mean[k] <= g_globals.btree->max_extents[k]) 
			{
                in++;
            }
        }

        if (in == g_globals.dataset->dims) {
            BITSET(node->flags,IS_BRUSHED);
            BITSET(node->flags,IS_BRUSHED_ANY);
        }

        return;
    }

    // Propagate markers up to parents
    //
    int brushed=0;
    bool brushed_any = false;
    for (int j=0; j<node->num_children; j++) 
	{
        if (BITTEST(g_globals.btree->nodes[node->child[j]].flags,IS_BRUSHED)) {
            brushed++;
        }
        if (BITTEST(g_globals.btree->nodes[node->child[j]].flags,IS_BRUSHED_ANY)) {
            brushed_any = true;
        }
    }

	if ( BrushSemantic == 0 ) /* ALL */
	{
		// All my children are marked, mark myself too
		//
		if (brushed == node->num_children)
		{
			BITSET(node->flags,IS_BRUSHED);
		}
	}
	else if ( BrushSemantic == 1 ) /* ANY */
	{
		// Any my children is marked, mark myself too
		//
		if ( brushed_any == true ) 
		{
			BITSET(node->flags,IS_BRUSHED);
		}
	}

	if ( brushed_any == true ) {
		BITSET(node->flags,IS_BRUSHED_ANY);
	}
}

void hierMarkBrushedClustersByColor(Cluster* node, double L, double R)
{
    BITCLEAR(node->flags,IS_BRUSHED);
    BITCLEAR(node->flags,IS_BRUSHED_ANY);

    for (int i=0; i<node->num_children; i++) {
        hierMarkBrushedClustersByColor(&(g_globals.btree->nodes[node->child[i]]), L, R);
    }

    // Reached the leaf node?
    //
    if (node->num_children == 0)
    {
        if (node->color>=L && node->color<=R)
        {
            BITSET(node->flags,IS_BRUSHED);
            BITSET(node->flags,IS_BRUSHED_ANY);

            // Setup min/max extents
            //
            for (int d=0; d<g_globals.dataset->dims; d++) 
			{
                if (g_globals.btree->min_extents[d] > node->mean[d]) 
					g_globals.btree->min_extents[d] = node->mean[d];

                if (g_globals.btree->max_extents[d] < node->mean[d]) 
					g_globals.btree->max_extents[d] = node->mean[d];
            }
        }

        return;
    }

    // Propagate markers up to parents
    //
    int brushed=0;
    bool brushed_any = false;
    for (int j=0; j<node->num_children; j++) {
        if (BITTEST(g_globals.btree->nodes[node->child[j]].flags,IS_BRUSHED)) {
            brushed++;
        }
        if (BITTEST(g_globals.btree->nodes[node->child[j]].flags,IS_BRUSHED_ANY)) {
            brushed_any = true;
        }
    }

	if ( BrushSemantic == 0 ) /* ALL */
	{
		// All my children are marked, mark myself too
		//
		if (brushed == node->num_children)
		{
			BITSET(node->flags,IS_BRUSHED);
		}
	}
	else if ( BrushSemantic == 1 ) /* ANY */
	{
		// Any my children is marked, mark myself too
		//
		if ( brushed_any == true ) 
		{
			BITSET(node->flags,IS_BRUSHED);
		}
	}

	if ( brushed_any == true ) {
		BITSET(node->flags,IS_BRUSHED_ANY);
	}
}

void hierModifyBrushParametersByExtents()
{
    for (int i=0; i<g_globals.dataset->dims; i++) 
	{
		double n = MAX2(g_globals.btree->min_extents[i], g_globals.dataset->all_dim_min[i]);
		if (g_globals.btree->min_extents[i] == HUGE_VAL)
			n = 0;
		double x = MIN2(g_globals.btree->max_extents[i], g_globals.dataset->all_dim_max[i]);
		if (g_globals.btree->max_extents[i] == -HUGE_VAL)
			x = 0;
		g_globals.curr_brush->SetMinMax(i, n, x);
//        g_globals.curr_brush->size[i] = g_globals.btree->max_extents[i] - g_globals.btree->min_extents[i];
//        g_globals.curr_brush->pos[i] = g_globals.btree->min_extents[i] + (g_globals.curr_brush->size[i]/2.);
    }
}

void hierModifyExtentsByBrushParameters()
{
    for (int i=0; i<g_globals.dataset->dims; i++) {
        g_globals.btree->min_extents[i] = g_globals.curr_brush->Min(i);
        g_globals.btree->max_extents[i] = g_globals.curr_brush->Max(i);
    }
}


void hierInitClusterColors(Cluster* node, double L, double R)
{
    node->color = .5*(L+R);

    /*SID*/ if (!node->num_children) return;

    double k = (R-L)/(double)(node->num_children);

    for (int i=0; i<node->num_children; i++) {
        hierInitClusterColors(&g_globals.btree->nodes[node->child[i]], L+i*k, L+(i+1)*k);
    }
}

void hierInitClusterColorsWithDistortion(Cluster* node, double L, double R)
{
    node->color = .5*(L+R);

    double color_l=L;
    double color_r=L;

    for (int i=0; i<node->num_children; i++)
    {
        double k = (R-L) * g_globals.btree->nodes[node->child[i]].entries/(double)node->entries;
        color_l = color_r;
        color_r = color_l + k;
        hierInitClusterColors(&g_globals.btree->nodes[node->child[i]], color_l, color_r);
    }
}

void hierDeleteClusterTree()
{
	if (g_globals.btree) { delete g_globals.btree; g_globals.btree = NULL; }
	CHierHist::cfHistoFinalize();
}


void hierInitClusterTree()
{
    if (g_globals.btree) return;

    g_globals.btree = new ClusterTree;

    int N = strlen(g_globals.dataset->datafile);
//jing
	char* str = new char[N+1];
//jing
    strcpy(str,g_globals.dataset->datafile);
//jing    strcpy(&str[N-4],".cf");
    strcpy(&str[N-4],".cg");

//jing
	if ( g_globals.hier_info->database )
	{
//jing for future changes		if (!g_globals.btree->ReadClusterTreeFileWithDatabase(str)) 
		if (!g_globals.btree->ReadClusterTreeFile(str)) 
		{
			hierDeleteClusterTree();
			return;
		}
	}
	else
	{
		if (!g_globals.btree->ReadClusterTreeFile(str)) 
		{
			hierDeleteClusterTree();
			return;
		}
	}

    hierInitClusterColors(&g_globals.btree->nodes[0], 0, 1);
    hierClearNodeFlags();
    hierModifyExtentsByBrushParameters();
    hierMarkBrushedClustersByExtents(&g_globals.btree->nodes[0]);

  

	CHierHist::cfHistoInit();
    cfNavInit();
    cfTreeMapInit();

    double root_radius = g_globals.btree->nodes[0].sqrt_radius;
    hierMarkContourNodes( root_radius*g_globals.hier_info->cluster_radius, IS_CONTOUR0 );
    hierMarkContourNodes( root_radius*g_globals.hier_info->brushed_radius, IS_CONTOUR1 );
   	g_globals.hier_info->root_radius = g_globals.btree->nodes[0].sqrt_radius;

	//jing
    //try
    hierMarkFlatBrushedClustersByExtents(g_globals.curr_brush, &g_globals.btree->nodes[0]);
	SetupBrushParameters();

	delete [] str;
}


void hierClearNodeFlags(int bits)
{
    for (int i=0;i<g_globals.btree->nodes.Len();i++) {
        Cluster &c = g_globals.btree->nodes[i];
        BITCLEAR(c.flags,bits);
    }
}

void hierMarkContourNodes(double contour_radius, int FLAG)
{
    // A node c is on contour if:
    // c.parent.radius > contour_radius AND
    //  (a)  c.radius <= contour_radius, OR
    //  (b)  c.radius > contour_radius AND c is a leaf
    //
    for (int i=0;i<g_globals.btree->nodes.Len();i++)
    {
        Cluster &c = g_globals.btree->nodes[i];

        BITCLEAR(c.flags,FLAG);

        double par_radius = HUGE_VAL;
        if (c.parent>=0) par_radius = g_globals.btree->nodes[c.parent].sqrt_radius;

        if (par_radius > contour_radius)
        {
            if (c.sqrt_radius <= contour_radius || c.num_children==0)
            {
                assert(c.color<=1.0 && c.color>=0);
                BITSET(c.flags,FLAG);
            }
        }
    }
}

void hierInitExtents()
{
    for (int d=0; d<g_globals.dataset->all_dims; d++) {
        g_globals.btree->min_extents[d] = HUGE_VAL;
        g_globals.btree->max_extents[d] = -HUGE_VAL;
    }
}


void hierSetMeanOnOff()
{
/*	g_globals.btree->ReadClusterTreeFile2();
    for (int b=0;b<g_globals.btree->nodes.Len();b++)
    {
        Cluster &c = g_globals.btree->nodes[b];
        int dims=0;
        for (int i=0;i<g_globals.dataset->all_dims;i++)
            if (g_globals.dataset->on[i]) {
                c.mean[dims] = c.mean[i];
                c.max[dims] = c.max[i];
                c.min[dims] = c.min[i];
                dims++;
            }
    }
*/
}


void hierReorderDimension()
{
/*	g_globals.btree->ReadClusterTreeFile3();
    for (int b=0;b<g_globals.btree->nodes.Len();b++)
    {
        Cluster &c = g_globals.btree->nodes[b];
        int dims=0;
        for (int i=0;i<g_globals.dataset->all_dims;i++)
            if (g_globals.dataset->on[i]) {
                c.mean[dims] = c.mean[i];
                c.max[dims] = c.max[i];
                c.min[dims] = c.min[i];
                dims++;
            }
    }
*/
}


//jing_flat_sb_start
static bool FlatSBtestCoverage(Brush *brush, Cluster* node)
{
    bool in = true;
	for ( int j =0; j < g_globals.dataset->dims; j++) 
	{
		if ( node->max[j] < brush->Min(j)) 
        {
            in = false;
            continue;
        }
		if ( node->min[j] > brush->Max(j)) 
        {
            in = false;
            continue;
        }
	}

    return in;
}

void hierMarkFlatBrushedClustersByExtents(Brush *brush, Cluster* node)
{
    node->flat_brushed = false;
    node->contain_flat_brushed = false;

    if ( FlatSBtestCoverage(brush, node) == true )
    {
        node->contain_flat_brushed = true;
    }
    else 
    {
        return;
    }

    if ( node->num_children == 0 || node->sqrt_radius <= g_globals.hier_info->brushed_radius )
        node->flat_brushed = true;
    else
    {
        for (int i=0; i<node->num_children; i++)
            hierMarkFlatBrushedClustersByExtents(brush, &(g_globals.btree->nodes[node->child[i]]));
    }
        return;
}

void HierChangeFlatBrushAccordingToClusters()
{
    int i;
    ARR<double> brush_min, brush_max;
    brush_min.SetLen(g_globals.dataset->dims);
    brush_max.SetLen(g_globals.dataset->dims);
    for ( i = 0; i < g_globals.dataset->dims; i++ )
    {
        brush_min[i] = g_globals.dataset->dim_max[i];
        brush_max[i] = g_globals.dataset->dim_min[i];
    }
    TraceJoinFSBBrushedWithClusters(g_globals.btree->nodes[0], brush_min, brush_max);
    for ( i = 0; i < g_globals.dataset->dims; i++ )
    {
        if ( brush_min[i] > brush_max[i] )
            return;
    }
    for ( i = 0; i < g_globals.dataset->dims; i++ )
    {
        g_globals.curr_brush->SetMinMax(i, brush_min[i], brush_max[i]);
    }
}


void JoinBrush( const Cluster &node, ARR<double> &brush_min, ARR<double> &brush_max)
{
    for ( int i = 0; i < g_globals.dataset->dims; i++ )
    {
        if ( brush_min[i] > node.min[i] )
            brush_min[i] = node.min[i] - 1e-5;
        if ( brush_max[i] < node.max[i] )
            brush_max[i] = node.max[i] + 1e-5;
    }
}

void TraceJoinFSBBrushedWithClusters( const Cluster &node, ARR<double> &brush_min, ARR<double> &brush_max)
{
    if ( !node.contain_flat_brushed )
        return;

    if ( node.flat_brushed )
    {
       JoinBrush(node, brush_min, brush_max);
    }
    else
    {
        for (int j=0; j<node.num_children; j++) 
            TraceJoinFSBBrushedWithClusters(g_globals.btree->nodes[node.child[j]], brush_min, brush_max);
    }
}

//jing_flat_sb_end
