#include <BrushStore.h>
#include <Brush.h>

using namespace cache;
/*
* This is a central repository that stores the history of the user
* brushes and can be used for prediction purposes both by the
* prefetcher and the replacement policy
*/
/**
* Stores the number of brushes to be stored in 
* the brush store
* \param size default size is 2
* \return 
*/
BrushStore::BrushStore(int size)
{
	this->maxSize = size;
	this->dequeBrush = new std::deque<Brush*>;
}

/**
* Copies the brush by value into the brush store
* \param *brush 
* \return 
*/
void BrushStore::addBrush(Brush *brush)
{
	//Using the copy constructor
	Brush* newBrush = new Brush(*brush);

	this->dequeBrush->push_front(newBrush);		
	if(dequeBrush->size() > maxSize)
	{
		//Removing the brush from the memory
		Brush *tempBrush = (*dequeBrush)[dequeBrush->size()-1];
		delete(tempBrush);
		dequeBrush->pop_back();
	}
}

/**
* Gets the ith brush from the brush store
* \param index 
* \return 
*/
Brush* BrushStore::getBrush(unsigned int index)
{
	Brush* resultBrush = NULL;
	if(index < dequeBrush->size())
		resultBrush = ((*dequeBrush)[index]);				
	return(resultBrush);
}

/**
* Destroys the BrushStore
* \return 
*/
BrushStore::~BrushStore()
{
	for(unsigned int i=0;i<dequeBrush->size();i++)
	{
		Brush* tempBrush = (*(this->dequeBrush))[i];

		delete(tempBrush);
	}
	delete(dequeBrush);
}



