#include <cache/ConfigFileParser.h>
#include <ConfigurationStore.h>
#include <fstream>
#include <cache/FileNotFoundException.h>


using namespace cache;

ConfigFileParser::ConfigFileParser(std::string fileName):fileName(fileName)
{}

CacheConfigStore* ConfigFileParser::parse()
{
	CacheConfigStore* configStore = new CacheConfigStore();
	std::ifstream inputStream(fileName.c_str(),std::ios::in);

	if(!inputStream.good())
		throw FileNotFoundException(fileName);

	//Reading the Cache Parameters
	std::string inputString;
	while(!inputStream.eof())
	{
		inputStream>>inputString;
		if(inputString == "RemainderQuery")
		{
			readQueryParameters(inputStream,*configStore);
		}
		if(inputString == "Cache")
		{
			readCacheParameters(inputStream,*configStore);
		}
		if(inputString == "Prefetcher")
		{
			readPrefetchParameters(inputStream,*configStore);
		}
		if(inputString == "MemoryIndex")
		{
			readIndexParameters(inputStream,*configStore);
		}			
	}
	return(configStore);
}

void ConfigFileParser::readQueryParameters(std::ifstream& inputStream,
										   CacheConfigStore &configStore)
{
	std::string propertyName;
	std::string value;

	inputStream>>propertyName>>value;
	if(propertyName == "Type")
	{
		configStore.queryType = value;								
	} 	
}

void ConfigFileParser::readCacheParameters(std::ifstream& inputStream,
										   CacheConfigStore &configStore)
{
	std::string propertyName;
	unsigned long value;

	inputStream>>propertyName>>value;
	if(propertyName == "SizeOfCache")
	{
		configStore.cacheSize = value;

	} // if(propertyName == "SizeOfCache")

	std::string replacementPolicy;
	inputStream>>propertyName>>replacementPolicy;

	if(propertyName == "ReplacementPolicy")
	{
		configStore.replacementPolicy = replacementPolicy;				
	} // if(propertyName == "ReplacementPolicy")

}

void ConfigFileParser::readPrefetchParameters(std::ifstream& inputStream,
											  CacheConfigStore &configStore)
{
	std::string value;
	std::string propertyName;		
	inputStream>>propertyName>>value;

	if(propertyName == "Type")
	{
		if(value == "Adaptive")
			configStore.prefetchType = 
			CacheConfigStore::Adaptive;
		if(value == "Fixed")
			configStore.prefetchType = 
			CacheConfigStore::Fixed;
	} 

	inputStream>>propertyName>>value;

	if(propertyName == "PrefetchStrategy")
	{

		if(value == "Noprefetch")
			configStore.prefetchStrategy = 
			CacheConfigStore::Noprefetch;

		if(value == "Random")
			configStore.prefetchStrategy = 
			CacheConfigStore::Random;

		if(value == "Direction")
			configStore.prefetchStrategy = 
			CacheConfigStore::Direction;

		if(value == "Focus")
			configStore.prefetchStrategy = 
			CacheConfigStore::Focus;

	} // if(propertyName == "ReplacementPolicy")

	unsigned int sizeOfBrushStore = 0;
	inputStream>>propertyName>>sizeOfBrushStore;

	if(propertyName == "SizeOfBrushStore")
	{
		configStore.sizeOfBrushStore = sizeOfBrushStore;
	}
}

void ConfigFileParser::readIndexParameters(std::ifstream& inputStream,
										   CacheConfigStore &configStore)
{
	std::string propertyName;
	std::string value;
	inputStream>>propertyName>>value;
	if(propertyName == "Type")
	{
		if(value=="RTree")
		{
			configStore.memoryIndex = "RTree";
			readRTreeParameters(inputStream,
				configStore);
		}
		else if(value == "SuperIndex"){
			configStore.memoryIndex = "SuperIndex";
			readRTreeParameters(inputStream,
								configStore);
		}
		else			
			configStore.memoryIndex = "Sequential";			
	} // if(propertyName == "Type")		
}

void ConfigFileParser::readRTreeParameters(std::ifstream& inputStream,
										   CacheConfigStore &configStore)
{

	std::string propertyName;
	unsigned int value;
	inputStream>>propertyName>>value;
	if(propertyName == "IndexCapacity")
	{
		configStore.indexCapacity = value; 				
	} 

	inputStream>>propertyName>>value;
	if(propertyName == "LeafCapacity")
	{
		configStore.leafCapacity = value; 				
	}

	double fillFactor;
	inputStream>>propertyName>>fillFactor;

	if(propertyName == "FillFactor")
	{
		configStore.fillFactor = fillFactor; 				
	}
}

ConfigFileParser::~ConfigFileParser(){}


