#include <cache/DistanceController.h>
#include <cache/CacheEntry.h>
#include <cache/Brush.h>
#include <cache/CacheEntryIterator.h>
#include <cache/BrushEntries.h>
#include <iostream>

using namespace cache;

DistanceController::DistanceController(IIndex *index,
									  unsigned long sizeOfBuffer):distLoader(index),
													  brushEntries(NULL),
													  currentBrush(NULL),
													  sizeOfBuffer(sizeOfBuffer)


{
	this->brushStore = new std::list<Brush*>;
	this->tempBrushStore = new std::list<Brush *>;
}

//Deleteing the brush store
DistanceController::~DistanceController()
{
	std::list<Brush*>::iterator it;
	for(it = brushStore->begin();it != brushStore->end();it++)
	{
		delete(*it);
	}
	for(it = tempBrushStore->begin();it != tempBrushStore->end();it++)
	{
		delete(*it);
	}

	delete(brushStore);
	delete(tempBrushStore);
	removeBrushEntries();
}
/*
* Register a new brush with the brush controller
*/
void DistanceController::addBrush(Brush *newBrush)
{
	Brush* b = new Brush(*newBrush);	
	if(b->getLeftExtent() < 0)
		std::cout<<"Error"<<std::endl;
	//Adds a new brush to the brush store.
	this->brushStore->push_back(b);
}

/*
 *	
 */
unsigned long DistanceController::getOccupiedCount()
{
	bool *check = new bool[sizeOfBuffer];
	for(unsigned int i =0;i<sizeOfBuffer;i++)
		check[i]= false;
	
	std::list<Brush*>::iterator it;
	unsigned int length = 0;
	for(it = brushStore->begin();it != brushStore->end();it++)
	{
		BrushEntries* brushEntries = this->distLoader.load(*it);
		int *ids;
		ids = brushEntries->getIDs(length);

		for(unsigned int i =0;i< length;i++)
		{
			check[ids[i]] = true;
		}
		delete[] ids;
		delete(brushEntries);
	}

	unsigned long count = 0;

	for(unsigned int i = 0;i<sizeOfBuffer;i++)
	{
		if(check[i]){
			count++;
		}
	}

	delete[] check;
	return(count);
}
/*
 *	Clear the state of the distance controller
 */
void DistanceController::clearState()
{	
	this->saveLeftOverBrush();
	std::list<Brush*>::iterator it;
	//Stores the temporary brush store in the permenant brush store.
	for(it = tempBrushStore->begin();it != tempBrushStore->end();it++)
		brushStore->push_back(*it);
	
	//Clear the temporary brush store
	tempBrushStore->clear();
	
	if(currentBrush != NULL)
		delete(currentBrush);
	if(brushEntries != NULL)
		delete(this->brushEntries);

	currentBrush = NULL;
	brushEntries = NULL;
}


/**
 * Stores the left over brush in the brushstore.
 * Note also takes care if the current brush and the 
 * the replaced brush are same.
 */
void DistanceController::saveLeftOverBrush()
{
	if(brushEntries != NULL  && currentBrush != NULL)
	{
		if(!(brushEntries->getBrush() == *currentBrush))
		{
			Brush *brush = this->brushEntries->getLeftOverBrush();

			if(brush != NULL) //If there is anything leftover from the brush
			{
				this->tempBrushStore->push_back(brush);
			}
		}
		else //If currentBrush == brushEntries brush
		{
			Brush* copyBrush = new Brush(*currentBrush);
			this->tempBrushStore->push_back(copyBrush);		
		}
		this->removeBrushEntries();
	}
}

/*
 *	Make sure that this returns NULL
 */
ICacheEntry* DistanceController::peek()
{
	ICacheEntry* iCacheEntry = NULL;
	//YOu might have to check if brushEntries = NULL;
	if(brushEntries != NULL)
		iCacheEntry = brushEntries->getNext();

	//If iCacheEntry = NULL then 
	while(iCacheEntry == NULL)
	{
		//Before you switch over to the next replace brush save whatever is left over.
		
		this->saveLeftOverBrush();

		this->removeBrushEntries();

		this->selectDistanceBrush(this->currentBrush);

		if(brushEntries == NULL) //Incremental Loading and all nodes are full
			return(NULL);

		iCacheEntry = brushEntries->getNext();
		//If there are no entries for the replace brush in the cache
		//then you can remove that entry from the brush store
	}

	return(iCacheEntry);
}

/**
 * Removes the brush entries from the distance controller
 */
void DistanceController::removeBrushEntries()
{
	if(this->brushEntries != NULL)
		delete(brushEntries);
	brushEntries = NULL;
}
/*
 *	
 */
void DistanceController::remove()
{
	//Erases the last entry peeked for in the brush.
	this->brushEntries->erase();
}

/*
* Returns a cache entry iterator
* The procedure does its best to order the entries as per
* the distance from the current brush
*/
CacheEntryIterator* DistanceController::getCacheEntryIterator(Brush *currentBrush)
{
	this->currentBrush = new Brush(*currentBrush);

	return(new CacheEntryIterator(this));
}


/**
 * Selects the most distant brush from the current brush
 * and initializes the brushentries with the most distant brush
 * \param *currentBrush 
 */
void DistanceController::selectDistanceBrush(Brush *currentBrush)
{
	std::list<Brush*>::iterator it;
	std::list<Brush*>::iterator distantBrush;

	double maxDistance = -1,currentDistance = -1;

	for(it = brushStore->begin();it != brushStore->end();it++)
	{
			currentDistance = calculateDistance(currentBrush,*it);
			if(currentDistance > maxDistance)
			{
				maxDistance = currentDistance;
				distantBrush = it;
			}
	}
	if(maxDistance == -1)//No Brush is selected, bursh store is empty
	{
		this->brushEntries = NULL;	
	}
	else
	{
		this->makeBrushEntry(*distantBrush);
		//Removing the brush from the brushstore.
		delete(*distantBrush);
		brushStore->erase(distantBrush);
	}
}

/*
/*
*	Selects the most distant brush
*/
/*void DistanceController::selectDistance(Brush *currentBrush)
{

//Clear the current brushEntries
if(brushEntries != NULL)
{
delete(brushEntries);
brushEntries = NULL;
}

//This makes sure that the current brush is not removed from the list
// The brush store always has to contain the current brush
if(replaceBrush == NULL)
{
Brush *b = this->brushStore->front();
this->setReplaceBrush(b);

//pops the front of the queue
if(!(*currentBrush  == *replaceBrush))
{
this->brushStore->pop_front();
delete(b);
}
//Loads the replace brush
this->brushEntries = this->distLoader.load(replaceBrush);

//Select the left or the right edge
//as of now selecting the left edge
this->brushEntries->setDirection(true);

}
else if(!(*currentBrush  == *replaceBrush))
{
//Selects the most distant brush
Brush *b = this->brushStore->front();
this->setReplaceBrush(b);	

//pops the front of the queue
if(!(*currentBrush  == *replaceBrush))
{
this->brushStore->pop_front();
delete(b);
}

//Loads the replace brush
this->brushEntries = this->distLoader.load(replaceBrush);

//Select the left or the right edge
//as of now selecting the left edge
this->brushEntries->setDirection(true);
}
else
{
//You should not delete this it is pointing inside the brushstore
delete(replaceBrush);
replaceBrush = NULL;
}	

if(brushStore->size() == 0)
std::cout<<"Something wrong"<<std::endl;

}*/

/**
 * Calculates the distance between the current and the candidate brush
 * \param currentBrush 
 * \param candidateBrush 
 * \return double
 */
double DistanceController::calculateDistance(Brush* currentBrush,
											 Brush* candidateBrush)
{
	Tools::Geometry::Region* currentRegion = 
			(Tools::Geometry::Region*) &currentBrush->getShape();

	Tools::Geometry::Region* candidateRegion = 
			(Tools::Geometry::Region*) &candidateBrush->getShape();

	Tools::Geometry::Point currentPoint,candidatePoint;

	currentRegion->getCenter(currentPoint);

	candidateRegion->getCenter(candidatePoint);

	double distance = currentPoint.getMinimumDistance(candidatePoint);
	
	return(distance);
}


void DistanceController::makeBrushEntry(Brush *brush)
{
	this->brushEntries = distLoader.load(brush);

	Tools::Geometry::Region* currentRegion = 
					(Tools::Geometry::Region*) &currentBrush->getShape();
	
	Tools::Geometry::Point centerPoint;
	currentRegion->getCenter(centerPoint);
	double xcenter = centerPoint.getCoordinate(0);

	double leftDistance = fabs(brush->getLeftExtent()-xcenter);
	double rightDistance = fabs(brush->getRightExtent()-xcenter);
	
	if(leftDistance >= rightDistance)
		//set Direction left to right
		this->brushEntries->setDirection(true);
	else
		this->brushEntries->setDirection(false);
	//set Direction right to left
}
