#include <cache/DistanceReplacement.h>
#include <cache/BrushEntries.h>
#include <cache/Brush.h>
#include <cache/Cache.h>
#include <cache/CacheEntryIterator.h>

using namespace cache;

		DistanceReplacement::DistanceReplacement(Cache *cache,
			CacheEntry<bool>* buffer,
			unsigned long sizeOfBuffer,
			IIndex* index): ReplacementPolicy(cache),//set the cache replacement policy
						   currentBrush(NULL),distController(index,sizeOfBuffer),
						   cacheEntryIterator(NULL)
		{					
			this->index = index;
			this->freeList = new DoublyList<CacheEntry<bool>*>;
			this->lBuffer = buffer;
			this->sizeOfBuffer = sizeOfBuffer;

			//Inserting all the nodes into the free list
			for(unsigned long i=0;i< this->sizeOfBuffer;i++)
			{
				DoublyList<CacheEntry<bool> *>::Node* node,tempNode;

				CacheEntry<bool>* cacheEntryPtr = &this->lBuffer[i];		
				//just to note that the cacheentry is now added to the list.
				cacheEntryPtr->replacementValue = true;
				//The node contains a pointer to the cacheEntry		
				node = new DoublyList<CacheEntry<bool>*>::Node(cacheEntryPtr);
				
				this->freeList->AppendNode(node);
			} // for(int i=0;i< this->sizeOfBuffer;i++)
			this->currentBrush = new Brush();
			this->tempBrush = new Brush();
		}


		DistanceReplacement::~DistanceReplacement()
		{
			delete(this->tempBrush);
			delete(this->currentBrush);
			delete(this->freeList);
		}

		ICacheEntry* DistanceReplacement::peek()
		{
			ICacheEntry* result = NULL;

			if(!freeList->isEmpty())
			{
				DoublyList<CacheEntry<bool>*>::Node* node = freeList->getFirst();
				result = node->nData;
			}
			/*Have to do all the distance stuff here*/
			else
			{
				ICacheEntry *iCacheEntry = NULL;

				if(this->cacheEntryIterator == NULL)
				{
					this->cacheEntryIterator = 
						this->distController.getCacheEntryIterator(this->currentBrush);
				}

				while((iCacheEntry = cacheEntryIterator->peek()) != NULL)
				{
					CacheEntry<bool>* cacheEntry  = 
						&this->lBuffer[iCacheEntry->getID()];

					if(cacheEntry->replacementValue == true)
					{
						//Remove the entry from being a candidate for replacement
						result = iCacheEntry;
						break;
					}
				}
		}
		peekResult = result;
		if(peekResult == NULL)
		{//That means incremental loading
			delete(this->cacheEntryIterator);
			cacheEntryIterator = NULL;
		}
		return(result);
	}


		ICacheEntry* DistanceReplacement::getVictimItem()
		{
			ICacheEntry* result = NULL;

			if(!freeList->isEmpty())
			{
				DoublyList<CacheEntry<bool>*>::Node* node = freeList->getFirst();

				result = node->nData;
				
				freeList->RemoveNode(node);

				//Deletes this node from the list
				delete(node);

				peekResult = result;
			}else
			{
				if(peekResult != NULL)//Remove the peeked result
					this->cacheEntryIterator->remove();
			}
			return(peekResult);
		}

		/*
		 *	Sets the replacement value to true
		 */
		void DistanceReplacement::servedItem(ICacheEntry& icacheEntry)
		{
			CacheEntry<bool>* cacheEntry = 
				&this->lBuffer[icacheEntry.getID()];
			
			cacheEntry->replacementValue = true;			
		}

		
		/*
		 *	This method is called each time we get a new brush
		 */
		void DistanceReplacement::brushModified(Brush& brush)
		{
			*this->tempBrush = brush;
		}
		/*
		 *	Note only is we have remainder queries
		 *  does it make sense to add the brush to the brush
		 *  to the brush store.
		 */
		void DistanceReplacement::ifRemainder(bool flag)
		{
			if(cacheEntryIterator != NULL)
			{
				delete(this->cacheEntryIterator);
				this->cacheEntryIterator = NULL;
			}
			if(flag)
			{
				*this->currentBrush = *tempBrush;
				this->distController.addBrush(currentBrush);
			}
		}
		//No you can implement this should not matter a lot.
		//Remove the entries and add them to the free list
		void DistanceReplacement::removeEntries(std::list<ICacheEntry*>& listCacheEntry)
		{
			std::list<ICacheEntry*>::iterator it;
			for(it =  listCacheEntry.begin();it != listCacheEntry.end();it++)
			{
				CacheEntry<bool>* che = &lBuffer[(*it)->getID()];

				DoublyList<CacheEntry<bool>*>::Node*  node = 
							new DoublyList<CacheEntry<bool>*>::Node(che);

				this->freeList->AppendNode(node);
			}
		}

		/*
		 * Sets the served item to false
		 */
		void DistanceReplacement::hitCacheEntry(ICacheEntry& icacheEntry)
		{
			CacheEntry<bool>* cacheEntry = 
						&this->lBuffer[icacheEntry.getID()];
			cacheEntry->replacementValue = false;
		}

		/*
		 *	Sets the served item to false
		 */
		void DistanceReplacement::newData(ICacheEntry &iCacheEntry)
		{
			CacheEntry<bool>* cacheEntry = 
				&this->lBuffer[iCacheEntry.getID()];
			cacheEntry->replacementValue = false;
		}


		void DistanceReplacement::clearCache()
		{

		}

		/**
		* Check if the replacement Policy has a track of all the nodes.
		*/
		void DistanceReplacement::checkForError()
		{
			unsigned long freeCount = 
						this->freeList->count();
			unsigned long occupiedCount = 
						this->distController.getOccupiedCount();

			unsigned long totalCount = freeCount + occupiedCount;

			if(totalCount < this->sizeOfBuffer)
			{
				std::cout<<"Error";
			}
		}
	
