// Type.cpp: implementation of the Type class.
//
//////////////////////////////////////////////////////////////////////

#pragma warning(disable:4786)

#include <meta/Type.h>
#include <meta/Interfaces.h>

using namespace std;

std::vector<Type::TypeData> Type::mTypes;

Type Type::fromName(const char* name)
{
	Type rval;
	for ( int i=0; i<(int)mTypes.size(); i++ ) {
		if ( !strcmp(mTypes[i].name,name) ) {
			rval.mTypeId = i;
			return rval;
		}
	}
	return rval;
}

const char* Type::getName() const
{
	return Type::mTypes[mTypeId].name;
}

Type Type::badType()
{
	return Type();
}

bool Type::isBad() const
{
	return (mTypeId==-1) ? true : false;
}

bool Type::isAbstract()
{
	return mIsAbstract;
}

Type Type::getParent() const
{
	Type rval;
	if ( mTypeId )
		rval.mTypeId = Type::mTypes[mTypeId].parent;
	return rval;
}

bool Type::isDerivedFrom(Type type) const
{
	if ( mTypeId == 0 ) return false;
	int parent = Type::mTypes[mTypeId].parent;
	if ( parent == type.mTypeId ) return true;
	Type ptype;
	ptype.mTypeId = parent;
	return ptype.isDerivedFrom(type);
}

bool Type::canCreateInstance() const
{
	return Type::mTypes[mTypeId].createMethod ? true : false;
}

IValue* Type::createInstance(const char *str) const
{
	return (Type::mTypes[mTypeId].createMethod)(str, Type::mTypes[mTypeId].data);
}

/*static*/ 
Type Type::registerAbstractType(Type parent,const char* name)
{
	Type rval = Type::fromName(name);
	if (rval.isBad()) {
		rval = registerType(parent, name, NULL, NULL);
		rval.mIsAbstract = true;
	}
	return rval;
}

/*static*/ 
Type Type::registerType(Type parent, const char* name,
						Type::CreateValueMethod createMethod /*= 0*/, void* data /*= 0*/)
{
	// if we already have this type registered,
	// then don't register and return the registered type
	Type rval = Type::fromName(name);
	if (rval.isBad()) {
		TypeData td;
		td.name = new char[strlen(name)+1];
		strcpy(td.name,name);
		td.createMethod = createMethod;
		td.parent = parent.mTypeId;
		
		mTypes.push_back(td);
		rval.mTypeId = mTypes.size()-1;
	}
	return rval;
}
